/* -*- Mode: C++ -*-
 * $Id: FCFooCoachDecision.cc,v 1.7 2002/09/04 14:48:05 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FCFooCoachDecision.cc
 * Unitname: Framework/example/FCFooCoach
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 2001-11-15
 * Last modified by $Author: s02davro $ $Date: 2002/09/04 14:48:05 $
 *
 *
 * HISTORY
 *
 * $Log: FCFooCoachDecision.cc,v $
 * Revision 1.7  2002/09/04 14:48:05  s02davro
 * Updated FCFoos desision making
 *
 * Revision 1.6  2002/08/31 15:17:17  s02davro
 * FCFoo now run coach with -coach option, changed the main loop
 *
 * Revision 1.5  2002/01/16 09:54:19  frehe
 * Minor changes.
 *
 * Revision 1.4  2002/01/15 17:20:44  frehe
 * Changed the message sent by the coach.
 *
 * Revision 1.3  2002/01/14 17:33:10  frehe
 * Fixed the freeform message from the coach.
 *
 * Revision 1.2  2001/12/20 14:58:55  frehe
 * Call the decide method in onVisualSensorData.
 *
 * Revision 1.1  2001/11/15 16:50:13  frehe
 * Added a coach agent
 *
 */

/**
 * @file FCFooCoachDecision.cc
 * Implementation of the class FCFooCoachDecision.
 *
 * @par Unit:
 *      Framework/example/FCFooCoach
 * $Revision: 1.7 $
 * $Author: s02davro $
 * $Date: 2002/09/04 14:48:05 $
 */

#include "FCFooCoachDecision.h"

#include <unistd.h>
#include <errno.h>

#include "ViewManager.h"
#include "ActuatorInterface.h"
#include "InitCommand.h"
#include "CoachCommand.h"
#include "rs_assert.h"
#include "rs_log.h"


RS_BEGIN_NAMESPACE

FCFooCoachDecision::FCFooCoachDecision(ActuatorInterface* const actuator_interface)
  : Decision(actuator_interface),
    viewManager(ViewManager::instance()),
    worldModelInterface(WorldModelInterface::instance()),
    facts(worldModelInterface->getWorldFacts()) {
}

FCFooCoachDecision::~FCFooCoachDecision() {
}

ServerCommand* FCFooCoachDecision::initAgent() {
  InitCommand* init_cmd = new InitCommand(facts->CP_team_name,
					  facts->SP_version,
					  false);
  return init_cmd;
}

bool FCFooCoachDecision::start() {
  RS_REQUIRE( initialized == true ); 
}

bool FCFooCoachDecision::decide() {
  static int say_counter = 10;
  
  const WorldModel wm = worldModelInterface->getWorld();
  
  for (std::vector<PlayerObject>::const_iterator i = wm.getTeammates().begin();
       i != wm.getTeammates().end(); ++i) 
    RS_LOG ( LA_COACH, "Coach (t): " 
	     << Time::currentTime
	     << "\n        "
	     << i->getUniformNumber()
	     << "\n        "
	     << i->getPosition().getPoint());
  for (std::vector<PlayerObject>::const_iterator i = wm.getOpponents().begin();
       i != wm.getOpponents().end(); ++i)
    RS_LOG ( LA_COACH, "Coach (o): " 
	     << Time::currentTime
	     << "\n        "
	     << i->getUniformNumber()
	     << "\n        "
	     << i->getPosition().getPoint());
  
  if ( say_counter == 0 ) {
    //      std::string msg = "(say (freeform "Test"))";
    //      actuatorInterface->addToPlan(&CoachCommand(msg));
    say_counter = 9;
  } else {
    --say_counter;
  }
  return true;
}

void FCFooCoachDecision::onInit() {
  CoachCommand *cc = new CoachCommand(std::string("(eye on)"));
  actuatorInterface->addToPlan(cc);
  delete cc;
}

void FCFooCoachDecision::onAfterSensorData() {
  decide();
}

RS_END_NAMESPACE
