/* -*- Mode: C++ -*-
 * $Id: FCFooDecision.cc,v 1.24 2002/09/04 14:48:05 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FCFooDecision.cc
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.24 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2002/09/04 14:48:05 $
 *
 *
 * HISTORY
 *
 * $Log: FCFooDecision.cc,v $
 * Revision 1.24  2002/09/04 14:48:05  s02davro
 * Updated FCFoos desision making
 *
 * Revision 1.23  2002/08/31 15:17:16  s02davro
 * FCFoo now run coach with -coach option, changed the main loop
 *
 * Revision 1.22  2002/08/02 22:03:38  s02davro
 * Changed FCFoo to test the modifications done (WorldModelInterface, Errors ...)
 *
 * Revision 1.21  2001/10/14 21:21:43  frehe
 * Added better support for messages from self and from other players.
 *
 * Revision 1.20  2001/10/14 14:41:42  frehe
 * Added a sample communication between the agent and itself.
 *
 * Revision 1.19  2001/09/26 13:05:37  frehe
 * Changed the order of include files to make circular updates less likely.
 *
 * Revision 1.18  2001/09/01 23:55:08  frehe
 * Made minor changes.
 *
 * Revision 1.17  2001/08/30 22:08:26  frehe
 * Wrote two simple decision makers that goes to the ball and scores.
 *
 * Revision 1.16  2001/08/29 16:31:19  frehe
 * Added some exception handling.
 *
 * Revision 1.15  2001/08/29 14:16:27  frehe
 * Made a simpler decision maker to avoid skills (until I get them to work).
 *
 * Revision 1.14  2001/08/29 08:38:34  petan117
 * Unreachable code found, marked with a FIXME
 *
 * Revision 1.13  2001/08/17 07:15:16  frehe
 * Made changes to compile RoboSoc with RCCParser-0.10.
 *
 * Revision 1.12  2001/01/08 13:41:40  frehe
 * Fixed a minor bug in FCFooDecision.
 *
 * Revision 1.11  2001/01/08 13:27:45  frehe
 * Fixed a minor bug in FCFooDecision.
 *
 * Revision 1.10  2000/11/08 09:41:10  frehe
 * Updated the FCFooDecision to use the Score skill.
 *
 * Revision 1.9  2000/11/02 16:33:45  frehe
 * Minor updates.
 *
 * Revision 1.8  2000/10/24 15:15:46  frehe
 * Made minor updates and tests.
 *
 * Revision 1.7  2000/10/19 00:05:35  frehe
 * Made minor changes.
 *
 * Revision 1.6  2000/10/18 23:07:11  frehe
 * Added a say message and a display of them.
 *
 * Revision 1.5  2000/10/11 18:40:29  frehe
 * Upgraded FCFoo.
 *
 * Revision 1.4  2000/10/10 11:55:37  frehe
 * Fixed FCFooDecision to work with the new skills.
 *
 * Revision 1.3  2000/10/08 17:34:11  frehe
 * Fixed FCFoo.
 *
 * Revision 1.2  2000/09/05 21:12:55  frehe
 * Made FCFoo compile with RoboSoc 2.5.0
 *
 * Revision 1.1.1.1  2000/08/31 13:26:05  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FCFooDecision.cc
 * Implementation of the class FCFooDecision.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.24 $
 * $Author: s02davro $
 * $Date: 2002/09/04 14:48:05 $
 */


#include "FCFooDecision.h"

#include <unistd.h>
#include <errno.h>

#include "GameView.h"
#include "BallView.h"
#include "AgentView.h"
#include "PlayerView.h"
#include "ActuatorInterface.h"
#include "ViewManager.h"
#include "view_constants.h"
#include "InitCommand.h"
#include "MoveCommand.h"
#include "SayCommand.h"
#include "DashCommand.h"
#include "KickCommand.h"
#include "TurnCommand.h"
#include "TurnNeckCommand.h"
#include "rs_assert.h"
#include "rs_log.h"

RS_BEGIN_NAMESPACE


FCFooDecision::FCFooDecision(ActuatorInterface* const actuator_interface)
  : Decision(actuator_interface),
    viewManager(ViewManager::instance()),
    worldModelInterface(WorldModelInterface::instance()),
    facts(worldModelInterface->getWorldFacts()),
    gameView(NULL),
    ballView(NULL),
    agentView(NULL),
    playerView(NULL),
    moveToPosition(Point(0,0), 50, facts->SP_kickable_margin, 5.0),
    aural_data(0) {
  getView(gameView, GAME_VIEW_ID);
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
}

FCFooDecision::~FCFooDecision() {
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
}

ServerCommand* FCFooDecision::initAgent() {
  InitCommand* init_cmd = new InitCommand(facts->CP_team_name, 
					  facts->SP_version, 
					  facts->CP_goalie);
  return init_cmd;
}

bool FCFooDecision::start() {
  RS_REQUIRE( initialized == true );
}

bool FCFooDecision::decide() {
  static int say_counter = 10;
  static Plan plan;

  // Example of a simple decision maker using skills.
  // If the ball's position is known, the agent will run
  // To that position, and try to kick the ball towards the 
  // opponents goal.
  
  if (findBall.applicable()) {
    RS_LOG ( LA_DECISION, "FindBall" );
    plan = findBall.generatePlan();
    actuatorInterface->addToPlan(plan.getNextStep());     
  }
  else if (gotoBall.applicable()) {
    RS_LOG ( LA_DECISION, "GotoBall" );
    plan = gotoBall.generatePlan();
    actuatorInterface->addToPlan(plan.getNextStep());     
  }
  else {

    // Example of more primitive decision making without using skills.
    ServerCommand* scmd = 0;
 
    const WorldModel *wm = &worldModelInterface->getWorld();
    AngleDegUE ball_dir = ballView->getRelativeDirection();
    AngleDegUE turn_dir = ball_dir - agentView->getBodyDirection();

    if ( gameView->isKickAllowed() && (ballView->getRelativeDistance() 
				       <= facts->SP_kickable_margin)) {
      // The ball is kickable, can we kick it to the ball.      
      // Get the direction to the opponents goal.
      AngleDegUE goal_dir = agentView->getPosition().getDirectionTo
	(facts->getTheirGoal().getPosition());
      // Kick the ball towards the goal.
      scmd = new KickCommand(50,(goal_dir - 
				 agentView->getBodyDirection()).getAngle());
    }
    else {
      // The game has not begun, do nothing.
      scmd = new TurnCommand(0);
    }
    if ( scmd != 0 ) {
      // Send the command to the actuators.
      actuatorInterface->addToPlan(scmd);
      delete scmd;
    }
  }    

// Example of communication.
//
//    if ( say_counter == 0 ) {
//      saycmd = new SayCommand("Hello" + toString(aural_data));
//      actuatorInterface->addToPlan(saycmd);
//      delete saycmd;
//      say_counter = 9;
//    } else {
//      --say_counter;
//    }  

  return true;
}

void FCFooDecision::onAuralCoachSensorData() {
  std::cerr << "I got " << agentView->getLastCoachMessage()
	    << " from the coach" << std::endl;
}

void FCFooDecision::onAuralRefereeSensorData() {
  std::cerr << "I got message from the referee." << std::endl;
}

void FCFooDecision::onAuralPlayerSensorData() {
  std::cerr << "I got " << agentView->getLastPlayerMessage()
	    << " from " << agentView->getLastPlayerMessageDirection()
	    << std::endl;
  ++aural_data;
}

void FCFooDecision::onAuralSelfSensorData() {
  std::cerr << "I got " << agentView->getLastSelfMessage()
	    << " from myself" << std::endl;
  ++aural_data;
}

void FCFooDecision::onInit() {
  // Position the players with some distance at the beginning of game.
  Point movePoint = Point(-10,-10);
  if (agentView->getUniformNumber().isKnown()) {
    UInt nr = agentView->getUniformNumber().getValue();
    switch (nr) {      
    case 1:
      movePoint = Point(-10, 0);
      break;
    case 2:
      movePoint = Point(-5, -15);
      break;
    case 3:
      movePoint = Point(-5, 15);
      break;
    case 4:
      movePoint = Point(-25, 10);
      break;
    case 5:
      movePoint = Point(-25, -10);
      break;
    case 6:
      movePoint = Point(-25, 20);
      break;
    case 7:
      movePoint = Point(-25, -20);
      break;
    case 8:
      movePoint = Point(-45, 10);
      break;
    case 9:
      movePoint = Point(-25, -10);
      break;
    case 10:
      movePoint = Point(-25, 20);
      break;
    case 11:
      movePoint = Point(-25, -20);
      break;
    default:
      movePoint = Point(-10,-10);
    }
  }
  MoveCommand* move_cmd = new MoveCommand(movePoint);
  actuatorInterface->addToPlan(move_cmd);
}

void FCFooDecision::onCommandWarning() {
  decide();
}

RS_END_NAMESPACE
