/* -*- Mode: C++ -*-
 *
 * IDENTIFICATION
 *
 * Filename: BeckhamPass.cc
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Group11
 * Last modified by $Author: aip11 $ $Date: 2002/10/04 12:10:11 $
 *
 */

/**
 * @file BeckhamPass.cc
 * Implementation file for the skill class BeckhamPass.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: Group11
 * $Date: 2002/10/04 12:10:11 $
 */


#include "BeckhamPass.h"
#include "AgentView.h"
#include "BallView.h"
#include "GameView.h"
#include "PlayerView.h"
#include "view_constants.h"
#include "KickCommand.h"
#include "WorldFacts.h"

RS_BEGIN_NAMESPACE


BeckhamPass::BeckhamPass()
  : Skill(),
    facts(WorldFacts::instance()),
    ballView(NULL),
    gameView(NULL),
    agentView(NULL),
    corridorAngle(AngleDeg(15)) {
  getView(ballView, BALL_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
} 

BeckhamPass::~BeckhamPass() {
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
}

Plan BeckhamPass::generatePlan() {
  static Command* cmd;
  Step step;
  
  
  Vector bestPassPlayerToGoal =Vector(bestPassPlayer->getPosition().getPoint(),
				      facts->getTheirGoal().getPosition());
  bestPassPlayerToGoal = 
    bestPassPlayerToGoal.setLength(bestPassPlayerToGoal.getLength() / 10);

  Vector passVector = Vector(agentView->getPosition().getPoint(), 
			     bestPassPlayer->getPosition().getPoint());

  if (bestPassPlayer->getSpeedVector().isKnown()) {
    passVector = passVector + bestPassPlayer->getSpeedVector().getVector();
  }
  
  //  passVector = passVector + bestPassPlayerToGoal;

  if (agentView->getBodyDirection().isKnown())
    cmd = new KickCommand(passVector.getLength()*3, 
			  passVector.getAngle() - 
			  agentView->getBodyDirection().getValue());
  else
    cmd = new KickCommand(passVector.getLength()*3, passVector.getAngle());


  step.addCommand(cmd);
  delete cmd;
  return Plan(step);
}

FuzzyBool BeckhamPass::persistent() {
  return FuzzyBool(0.0);
}

FuzzyBool BeckhamPass::applicable() {
  
  //if player has the ball
  if (ballView->isBallKickable()) {
    
    bestPassPlayer = NULL;
    float bestPassValue = -1000;
    bool passOk;

    for (std::vector<PlayerObject>::const_iterator 
	   tm=playerView->getTeammates().begin(); 
	 tm != playerView->getTeammates().end(); 
	 ++tm) {
      
      passOk = true;
      
      if (tm->getPosition().isKnown() && agentView->getPosition().isKnown()) {
	for (std::vector<PlayerObject>::const_iterator 
	       op=playerView->getOpponents().begin(); 
	     op != playerView->getOpponents().end(); 
	     ++op) {
	  
	  if (op->getPosition().isKnown()) {
	    
	    Vector opVector = Vector(agentView->getPosition().getPoint(), 
				     op->getPosition().getPoint());
	    Vector tmVector = Vector(agentView->getPosition().getPoint(), 
				     tm->getPosition().getPoint());
	    
	    // all opponents must be outside pass corridor angle 
	    // while closer to the ball then the teammate.
	    if (abs(tmVector.getAngle() - opVector.getAngle()) < 
		corridorAngle &&
		opVector.getLength() < 
		tmVector.getLength() + opVector.getLength() * 0.05) {
	      passOk = false;
	      break;
	    }
	  }
	}
	// will pass if opponents found with unknown position 
	if (passOk && tm->getPosition().getPoint().getX() > bestPassValue) {
	  bestPassPlayer = &(*tm);
	  bestPassValue = tm->getPosition().getPoint().getX();
	}
      }
    }// end for
    if (bestPassPlayer != NULL) {
      return FuzzyBool(1.0);
    }
  }
  return FuzzyBool(0.0);
}


RS_END_NAMESPACE
