/* -*- Mode: C++ -*-
 *
 * IDENTIFICATION
 *
 * Filename: BeckhamPass.h
 * Unitname: Framework/skill
 * $Revision: 1.1 $
 * Created by: Alexander och Sam
 */

/**
 * @file BeckhamPass.h
 * Include file for the skill class FindBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: aip11 $
 * $Date: 2002/10/03 10:03:52 $
 */


#ifndef BECKHAMPASS_H
#define BECKHAMPASS_H


#include "Skill.h"
#include "PlayerObject.h"
#include "WorldFacts.h"

RS_BEGIN_NAMESPACE

class BallView;
class GameView;
class AgentView;
class PlayerView;

/**
 * @class FindBall FindBall.h
 * This class encapsulates a beckhamPass ball between teammates skill.
 *
 * <!--
 * @pre The preconditions are that we can excpect visual information in
 * The next cycle.
 * @post We find the ball. Or keep on turning
 *
 * @todo This needs to be done...
 * -->
 * $Revision: 1.1 $
 * $Author: aip11 $
 * $Date: 2002/10/03 10:03:52 $
 *
 */

class BeckhamPass : public Skill {
public:
  BeckhamPass ();
  virtual ~BeckhamPass();

  virtual Plan generatePlan();

  virtual FuzzyBool persistent();
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed() { return FuzzyBool(0.5); };

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed() { return FuzzyBool(0.5); };

  
protected:
  const WorldFacts* facts;
  BallView* ballView;
  GameView* gameView;
  AgentView* agentView;
  PlayerView* playerView;

private:
  const PlayerObject* bestPassPlayer;
  const AngleDeg corridorAngle;
};


RS_END_NAMESPACE


#endif // !BECKHAMPASS_H


