/* -*- Mode: C++ -*-
 * $Id: CatchBall.cc,v 1.5 2002/09/04 13:26:38 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CatchBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.5 $
 * Created by: Bjrn, Ola och Jenny
 * Last modified by $Author: s02davro $ $Date: 2002/09/04 13:26:38 $
 *
 *
 * HISTORY
 *
 *
 */

/**
 * @file CatchBall.cc
 * This is a definition file for the skill CatchBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2002/09/04 13:26:38 $
 */

#include "CatchBall.h"
#include "CatchCommand.h"
#include "TurnCommand.h"
#include "WorldFacts.h"
#include "AgentView.h"
#include "BallView.h"
#include "view_constants.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

CatchBall::CatchBall()
  : facts(WorldFacts::instance()),
    ballView(NULL),
    agentView(NULL) {
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
}

// --- DESTRUCTOR ---

CatchBall::~CatchBall() {
    viewManager->releaseView(BALL_VIEW_ID);
    viewManager->releaseView(AGENT_VIEW_ID);
}

// --- METHODS ---

Plan CatchBall::generatePlan() {
  static Command* cmd;
  Step step;
  
  cmd = new CatchCommand((ballView->getRelativeDirection() - 
			  agentView->getBodyDirection()).getDeg());  
  step.addCommand(cmd);  
  delete cmd;
  return Plan(step);
}

FuzzyBool CatchBall::persistent() {
  return FuzzyBool(0.0);
}


FuzzyBool CatchBall::applicable() {
  return (agentView->isGoalie() &&
	  ballView->getPosition().isKnown() && 
	  agentView->getPosition().isKnown() &&
	  agentView->getBodyDirection().isKnown() &&
	  ballView->isBallCatchable());
}


RS_END_NAMESPACE
