/* -*- Mode: C++ -*-
 * $Id: Dribble.h,v 1.4 2002/10/02 14:03:16 aip7 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Dribble.h
 * Unitname: Framework/skill
 * $Revision: 1.4 $
 * Created by: Anders och Erik
 * Last modified by $Author: aip7 $ $Date: 2002/10/02 14:03:16 $
 *
 *
 * HISTORY
 * 0.1 Initial version
 *
 */

/**
 * @file Dribble.h
 * This is the definitionfile of the skill Dribble.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: aip7 $
 * $Date: 2002/10/02 14:03:16 $
 */

#ifndef DRIBBLE_H
#define DRIBBLE_H


#include "machine_dependent_compiler_directives.h"
#include "Skill.h"

RS_BEGIN_NAMESPACE

class BallView;
class KickBallView;
class AgentView;
class PlayerView;
class WorldFacts;

/**
 * @class Dribble Dribble.h
 * The skill will try to dribble.
 * If the ball is in control range, this skill will try to dribble.
 *
 * @pre The preconditions for the skill are:
 * The ball is within control range. 
 * @post The intended postconditions for the skill are:
 * The the ball has moved towards the specified point.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: aip7 $
 * $Date: 2002/10/02 14:03:16 $
 */

class Dribble : public Skill {
public:
  Dribble(const Point def_dribblePoint = Point(22.0, 0.0),
	  const Float def_minDistanceTo = Float(10.0));

  /** Destructor should clean up after the skill. */
  virtual ~Dribble();

  /** This method generates a plan for how to dribble.
      The skill is assumed to be applicable.
      The plan is always one step long containing a kick-command. */
  virtual Plan generatePlan();

  /**
   * This method is used for setting the dribble point, i.e. the
   * point where you want the dribble to head for.
   */
  virtual void setDribblePoint(const Point point);

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();
  
  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed() { return FuzzyBool(0.5); };

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed() { return FuzzyBool(0.5); };

  
private:
  /** Hide copy-constructor. */
  Dribble(const Dribble&);

  /** Hide assignment operator. */
  Dribble& operator= (const Dribble&);

protected:
  const WorldFacts *facts;
  BallView *ballView;
  AgentView *agentView;
  PlayerView *playerView;
  KickBallView *kickBallView;
  Point dribblePoint;
  Float minDistanceTo;
};


RS_END_NAMESPACE

#endif // !DRIBBLE_H
