/* -*- Mode: C++ -*-
 * $Id: DribbleToPosition.h,v 1.8 2002/10/09 15:15:36 aip8 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DribbleToPosition.h
 * Unitname: Framework/skill
 * $Revision: 1.8 $
 * Created by:
 * Last modified by $Author: aip8 $ $Date: 2002/10/09 15:15:36 $
 *
 *
 * HISTORY
 * Revision 0.0  2000/09/05 21:12:02  Rosario, Marinus, Philip
 *
 * #include rs_log.h
 *
 */

/**
 * @file DribbleToPosition.h
 * Include file for the skill class DribbleToPosition.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.8 $
 * $Author: aip8 $
 * $Date: 2002/10/09 15:15:36 $
 */


#ifndef DRIBBLE_TO_POSITION_H
#define DRIBBLE_TO_POSITION_H

#define DRIBBLE_TO_POSITION_SECTORS 5
#define DRIBBLE_TO_POSITION_MAXTURNANGLE 45
#define DRIBBLE_TO_POSITION_MAXRELBODYANGLE 90

#define DRIBBLE_TO_POSITION_NOSKILL 0
#define DRIBBLE_TO_POSITION_KICKTOCONTROLBALLSKILL 1
#define DRIBBLE_TO_POSITION_MOVEWITHBALLTOPOSITIONSKILL 2

#include "SkillWithPosition.h"
#include "WorldFacts.h"
#include "KickToControlBall.h"
#include "MoveWithBallToPosition.h"
#include "MoveWithBallToPosition2.h"

RS_BEGIN_NAMESPACE

class BallView;
class GameView;
class AgentView;
class PlayerView;

/**
 * @class DribbleToPosition DribbleToPosition.h
 * This class encapsulates a find ball skill.
 *
 * <!--
 * @pre The preconditions are that we can excpect visual information in
 * The next cycle.
 * @post We find the ball. Or keep on turning
 *
 * @todo This needs to be done...
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.8 $
 * $Author: aip8 $
 * $Date: 2002/10/09 15:15:36 $
 */
class DribbleToPosition : public SkillWithPosition {
public:
  DribbleToPosition(const Point& pos);
  virtual ~DribbleToPosition();

  virtual Plan generatePlan();

  virtual FuzzyBool persistent();
  virtual FuzzyBool applicable();
  virtual FuzzyBool succeed();
  virtual FuzzyBool failed();

private:

	// Point* findNextPosition(Point* currentPosition /*, UInt maxSteps*/);

protected:
  const WorldFacts* worldFacts;
  BallView* ballView;
  GameView* gameView;
  AgentView* agentView;
  PlayerView* playerView;

  Point nextPositionToMoveTo;
	Float nextSpeed;

  KickToControlBall kickToControlBallSkill;
  MoveWithBallToPosition2 moveWithBallToPositionSkill;

private:
  bool planFinished;
  bool dribblingFailed;

};


RS_END_NAMESPACE


#endif // !DRIBBLE_TO_POSITION_H
