/* -*- Mode: C++ -*-
 * $Id: FindBall.cc,v 1.16 2002/09/30 14:27:16 aip2 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FindBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.16 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: aip2 $ $Date: 2002/09/30 14:27:16 $
 *
 *
 * HISTORY
 * $Log: FindBall.cc,v $
 * Revision 1.16  2002/09/30 14:27:16  aip2
 * Updated Pass.cc
 *
 * Revision 1.15  2002/09/04 13:27:23  s02davro
 * Findball don't turn unless we get new see information next cycle
 *
 * Revision 1.14  2002/09/04 07:56:07  s02davro
 * Findballfix + MoveToPosition
 *
 * Revision 1.13  2002/09/03 10:58:17  s02davro
 * FindBall update
 *
 * Revision 1.12  2002/08/02 22:02:30  s02davro
 * Changed some skills to test the modified views and WorldModelInterface.
 *
 * Revision 1.11  2001/09/29 18:55:20  frehe
 * Removed TRUE, FALSE, and UNKNOWN.
 *
 * Revision 1.10  2001/09/26 11:41:25  frehe
 * Update after move of TRUE, FALSE and UNKNOWN to inside FuzzyBool.
 *
 * Revision 1.9  2001/08/30 22:56:15  frehe
 * Replaced TRUE with FuzzyBool(1.0) since the static consts are not initialized properly.
 *
 * Revision 1.8  2001/08/23 00:31:58  frehe
 * Removed dependencies.
 *
 * Revision 1.7  2001/01/24 07:00:39  olalu526
 * Fine tuning and fixing.
 *
 * Revision 1.6  2001/01/11 22:00:19  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.5  2000/10/07 15:07:36  matka134
 * Now using the predicate IsBallKnown
 *
 * Revision 1.4  2000/09/29 17:55:02  olalu526
 * Added FaceDirection, fixed some in FindBall (now works as expected), fixed segmentation fault in TurnDirection and more.
 *
 * Revision 1.3  2000/09/28 18:52:17  olalu526
 * Now the agent works quite fine.
 *
 * Revision 1.2  2000/09/13 01:38:59  frehe
 * Updated the skills to use getView instead of static_cast<...>.
 *
 * Revision 1.1  2000/09/05 21:12:01  frehe
 * #include rs_log.h
 *
 */

/**
 * @file FindBall.cc
 * Implementation file for the skill class FindBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.16 $
 * $Author: aip2 $
 * $Date: 2002/09/30 14:27:16 $
 */


#include "FindBall.h"
#include "AgentView.h"
#include "BallView.h"
#include "GameView.h"
#include "view_constants.h"
#include "TurnCommand.h"
#include "TurnNeckCommand.h"

RS_BEGIN_NAMESPACE

 
FindBall::FindBall()
  : Skill(),
    ballView(NULL),
    gameView(NULL),
    agentView(NULL) {
  getView(ballView, BALL_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
} 

FindBall::~FindBall() {
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
}

Plan FindBall::generatePlan() {
  static Command* cmd;
  Step step;

  // Align neck with body
  cmd = new TurnNeckCommand((agentView->getBodyDirection() - 
			     agentView->getNeckDirection()).getDeg());
  step.addCommand(cmd);
  delete cmd;  

  if (gameView->getNextSeeTime() == gameView->getNextTime()) {
    
    // Turn an appropriate angle from last face direction.
    AngleDeg turnAngle;
    switch (agentView->getViewWidth()) {
    case VW_NARROW: 
      turnAngle = 45;
      break;
    case VW_NORMAL:
      turnAngle = 90;
      break;
    case VW_WIDE:
      turnAngle = 180;
      break;
    }  
    cmd = new TurnCommand((agentView->getNeckDirection() + turnAngle -
			   agentView->getBodyDirection()).getDeg());
  }
  else {
    
    // No visual information next cycle, wait.
    cmd = new TurnCommand((agentView->getNeckDirection() - 
			   agentView->getBodyDirection()).getDeg());
  }

  step.addCommand(cmd);
  delete cmd;
  return Plan(step);
}

FuzzyBool FindBall::persistent() {
  return FuzzyBool(0.0);
}

FuzzyBool FindBall::applicable() {
  return ((ballView->getPosition().isUnknown()) &&
	  (agentView->getBodyDirection().isKnown()) &&
	  (agentView->getNeckDirection().isKnown()));
}


RS_END_NAMESPACE
