/* -*- Mode: C++ -*-
 * $Id: FindBall.h,v 1.5 2002/09/03 10:58:17 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FindBall.h
 * Unitname: Framework/skill
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2002/09/03 10:58:17 $
 *
 *
 * HISTORY
 * $Log: FindBall.h,v $
 * Revision 1.5  2002/09/03 10:58:17  s02davro
 * FindBall update
 *
 * Revision 1.4  2001/09/01 23:51:45  frehe
 * Updated the skill with the new abstract skill class and the new version of getValue and getAngle.
 *
 * Revision 1.3  2001/08/23 00:31:58  frehe
 * Removed dependencies.
 *
 * Revision 1.2  2000/09/29 17:55:02  olalu526
 * Added FaceDirection, fixed some in FindBall (now works as expected), fixed segmentation fault in TurnDirection and more.
 *
 * Revision 1.1  2000/09/05 21:12:02  frehe
 * #include rs_log.h
 *
 */

/**
 * @file FindBall.h
 * Include file for the skill class FindBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2002/09/03 10:58:17 $
 */


#ifndef FINDBALL_H
#define FINDBALL_H


#include "Skill.h"

RS_BEGIN_NAMESPACE

class BallView;
class GameView;
class AgentView;

/**
 * @class FindBall FindBall.h
 * This class encapsulates a find ball skill.
 *
 * <!--
 * @pre The preconditions are that we can excpect visual information in
 * The next cycle.
 * @post We find the ball. Or keep on turning
 *
 * @todo This needs to be done...
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2002/09/03 10:58:17 $
 */
class FindBall : public Skill {
public:
  FindBall();
  virtual ~FindBall();

  virtual Plan generatePlan();

  virtual FuzzyBool persistent();
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed() { return FuzzyBool(0.5); };

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed() { return FuzzyBool(0.5); };

  
protected:
  BallView* ballView;
  GameView* gameView;
  AgentView* agentView;
};


RS_END_NAMESPACE


#endif // !FINDBALL_H
