/* -*- Mode: C++ -*-
 * $Id: FreeKick.h,v 1.2 2002/10/14 10:52:11 aip9 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FreeKick.h
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-07
 * Last modified by $Author: aip9 $ $Date: 2002/10/14 10:52:11 $
 *
 *
 * HISTORY
 *
 * $Log: FreeKick.h,v $
 * Revision 1.2  2002/10/14 10:52:11  aip9
 * Update skill
 *
 * Revision 1.1  2002/09/27 15:41:02  aip9
 * Added the FreeKick and FreeKickPositioning skill
 *
 * Revision 1.5  2001/09/06 13:13:39  andbo928
 * Added class specifier to a lot of methods. Added succeed and failed methods to cc file.
 *
 * Revision 1.4  2001/09/01 14:45:12  frehe
 * Updated the skill stub to the new abstract Skill class.
 *
 * Revision 1.3  2000/09/16 18:39:39  frehe
 * Fixed the log messages.
 *
 * Revision 1.2  2000/09/16 18:36:48  frehe
 * Made minor fixes the skill stubs.
 *
 * Revision 1.1  2000/09/07 16:10:34  frehe
 * Added stubs for the skills.
 *
 */

/**
 * @file FreeKick.h
 * Include file for the skill class FreeKick.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: aip9 $
 * $Date: 2002/10/14 10:52:11 $
 */


#ifndef FREEKICK_H
#define FREEKICK_H


#include "Skill.h"
#include "AgentView.h"
#include "PlayerView.h"
#include "BallView.h"
#include "GameView.h"
#include "WorldFacts.h"


RS_BEGIN_NAMESPACE


class AgentView;
class PlayerView;
class BallView;
class GameView;
class WorldFacts;


/**
 * @class FreeKick FreeKick.h
 * The first sentence is a brief description of the skill class FreeKick.
 * The rest is a longer description of the skill class FreeKick.
 * Use it to describe the skill!
 *
 * @pre The preconditions for the skill are ...
 * @post The intended postconditions for the skill are...
 *
 * @todo Things that should be done...
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: aip9 $
 * $Date: 2002/10/14 10:52:11 $
 */
class FreeKick : public Skill {
public:
	/** Default constructor. Here you can initialize the skill. */
	FreeKick(const Float32 def_ballPlayerDistance = 0.2,
		const Int32 def_maxTime = 190,
		const Int32 def_dash = 50,
		const Float32 def_maxDistance = 0.2, 
		const AngleDeg def_minTurnAngle = AngleDeg(5.0));

	FreeKick(const Point def_target,
		const Float32 def_ballPlayerDistance,
		const Int32 def_maxTime,
		const Int32 def_dash,
		const Float32 def_maxDistance, 
		const AngleDeg def_minTurnAngle);

	/** Destructor should clean up after the skill. */
	virtual ~FreeKick();

	/** This method is used to generate a plan for how the goal of
	the skill should be obtained by the agent. You can assume
	that the preconditions of the skill are true (this can be tested
	by calling the applicable() method, after you have implemented it). */
	virtual Plan generatePlan();

	/** This method should return a FuzzyBool telling the caller
	if the skill wants to be called again. */
	virtual FuzzyBool persistent();

	/** This method should implement the precondition specified in
	the description of the skill. */
	virtual FuzzyBool applicable();

	/** Return @c true (close to 1) if the skill has succeeded,
	i.e. its post conditions are satisfied. */
	virtual FuzzyBool succeed();

	/** Return @c true (close to 1) if the skill has failed,
	i.e. something went wrong. */
	virtual FuzzyBool failed();
  
private:
	/** Hide copy-constructor. */
	FreeKick(const FreeKick&);

	/** Hide assignment operator. */
	FreeKick& operator= (const FreeKick&);

protected:
	const WorldFacts * facts;
	AgentView * agentView;
	PlayerView * playerView;
	BallView * ballView;
	GameView * gameView;
  
	Time startTime;
	Int32 state;
	Float32 ballPlayerDistance;
	Int32 maxTime;
	Int32 dash;
	Float32 maxDistance;
	AngleDeg minTurnAngle;
	Point target;
};


RS_END_NAMESPACE


#endif // !FREEKICK_H
