/* -*- Mode: C++ -*-
 * $Id: FreeKickPositioning.cc,v 1.2 2002/10/14 10:52:11 aip9 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FreeKickPositioning.cc
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-07
 * Last modified by $Author: aip9 $ $Date: 2002/10/14 10:52:11 $
 *
 *
 * HISTORY
 *
 * $Log: FreeKickPositioning.cc,v $
 * Revision 1.2  2002/10/14 10:52:11  aip9
 * Update skill
 *
 * Revision 1.1  2002/09/27 15:41:03  aip9
 * Added the FreeKick and FreeKickPositioning skill
 *
 * Revision 1.5  2002/09/13 14:48:56  s02davro
 * FALSE -> FuzzyBool(0.0), TRUE -> FuzzyBool(1.0)
 *
 * Revision 1.4  2001/09/06 13:13:39  andbo928
 * Added class specifier to a lot of methods. Added succeed and failed methods to cc file.
 *
 * Revision 1.3  2000/09/16 18:39:39  frehe
 * Fixed the log messages.
 *
 * Revision 1.2  2000/09/16 18:36:48  frehe
 * Made minor fixes the skill stubs.
 *
 * Revision 1.1  2000/09/07 16:10:34  frehe
 * Added stubs for the skills.
 *
 */

/**
 * @file FreeKickPositioning.cc
 * Implementation file for the skill class FreeKickPositioning.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: aip9 $
 * $Date: 2002/10/14 10:52:11 $
 */


#include "FreeKickPositioning.h"
#include "view_constants.h"


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

FreeKickPositioning::FreeKickPositioning()
{
	getView(gameView, GAME_VIEW_ID);
	getView(ballView, BALL_VIEW_ID);
}


// --- DESTRUCTOR ---

FreeKickPositioning::~FreeKickPositioning()
{
	viewManager->releaseView(GAME_VIEW_ID);
	viewManager->releaseView(BALL_VIEW_ID);
}


// --- METHODS ---

Plan FreeKickPositioning::generatePlan()
{
	// Return an empty plan.
	return Plan();
}


FuzzyBool FreeKickPositioning::persistent()
{
	// Never persistent.
	return FuzzyBool(0.0);
}


FuzzyBool FreeKickPositioning::applicable()
{
	if (gameView->getPlayMode() == PM_OUR_FREE_KICK)
	{
		if (ballView->getPosition().isKnown())
		{
			return FuzzyBool(1.0);
		}
	}

	return FuzzyBool(0.0);
}

FuzzyBool FreeKickPositioning::succeed()
{
	// Always succedes.
	return FuzzyBool(0.0);
}

FuzzyBool FreeKickPositioning::failed()
{
	// Never fails.
	return FuzzyBool(0.0);
}


RS_END_NAMESPACE
