/* -*- Mode: C++ -*-
 * $Id: GoaliePositioning.cc,v 1.18 2002/09/30 19:07:38 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GoaliePositioning.cc
 * Unitname: Framework/skill
 * $Revision: 1.18 $
 * Created by: Fredrik Heintz 2000-09-07
 * Last modified by $Author: s02davro $ $Date: 2002/09/30 19:07:38 $
 *
 *
 * HISTORY
 *
 * $Log: GoaliePositioning.cc,v $
 * Revision 1.18  2002/09/30 19:07:38  s02davro
 * Bugfix of GoaliePositioning works ok now
 *
 * Revision 1.17  2002/09/26 15:27:08  s02davro
 * Small bugfix, still not ok
 *
 * Revision 1.16  2002/09/23 10:27:12  s02davro
 * Works ok, needs more testing and finetune
 *
 * Revision 1.15  2002/09/21 20:39:14  s02davro
 * Added the modified versions of GoaliePositioning, some bugs remain.
 *
 * Revision 1.1  2002/09/04 13:36:56  s02davro
 * Added old skills in /skills/old
 *
 * Revision 1.13  2001/11/29 14:57:32  aip9
 * Fixed a bug in applicable.
 *
 * Revision 1.12  2001/10/22 16:32:51  aip11
 * Added backward moving to keep ball in view or for quicker movement, rush out more aggresivly,changed intercept extrapolation point
 *
 * Revision 1.11  2001/10/11 11:05:04  aip11
 * Changed some Plan constructors to use heap instead of stack
 *
 * Revision 1.10  2001/10/10 18:33:43  aip11
 * Added some comment and tweaked a few constants
 *
 * Revision 1.9  2001/09/29 18:55:20  frehe
 * Removed TRUE, FALSE, and UNKNOWN.
 *
 * Revision 1.8  2001/09/26 11:41:25  frehe
 * Update after move of TRUE, FALSE and UNKNOWN to inside FuzzyBool.
 *
 * Revision 1.7  2001/09/26 09:59:52  aip11
 * Removed some couts
 *
 * Revision 1.6  2001/09/26 09:03:52  aip11
 * Fixed confidence bug
 *
 * Revision 1.4  2001/09/20 12:59:48  aip11
 * Somewhat working version
 *
 * Revision 1.3  2001/09/18 16:28:29  aip11
 * *** empty log message ***
 *
 * Revision 1.2  2001/09/06 13:12:53  andbo928
 * Added some methods to the class. (SkillStub was bad)
 *
 * Revision 1.1  2001/09/05 14:41:03  stejo127
 * Added the GoaliePositioning skill.
 *
 * Revision 1.3  2000/09/16 18:39:39  frehe
 * Fixed the log messages.
 *
 * Revision 1.2  2000/09/16 18:36:48  frehe
 * Made minor fixes the skill stubs.
 *
 * Revision 1.1  2000/09/07 16:10:34  frehe
 * Added stubs for the skills.
 *
 */

/**
 * @file GoaliePositioning.cc
 * Implementation file for the skill class GoaliePositioning.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.18 $
 * $Author: s02davro $
 * $Date: 2002/09/30 19:07:38 $
 */


#include "GoaliePositioning.h"
#include "AgentView.h"
#include "BallView.h"
#include "PlayerView.h"
#include "GameView.h"
#include "MoveToPosition.h"
#include "FindBall.h"
#include "WorldFacts.h"
#include "CatchCommand.h"
#include "KickCommand.h"
#include "TurnCommand.h"
#include "TurnNeckCommand.h"
#include "view_constants.h"


RS_BEGIN_NAMESPACE

// --- CONSTRUCTORS ---

GoaliePositioning::GoaliePositioning()
  : Skill(),    
    ballView(NULL),
    agentView(NULL),
    playerView(NULL),
    gameView(NULL),
    facts(WorldFacts::instance()) {
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
}


// --- DESTRUCTOR ---

GoaliePositioning::~GoaliePositioning() {
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
}

// --- METHODS ---   

Plan GoaliePositioning::generatePlan() {
  PointUE ballPos = ballView->getPosition();
  VectorUE ballSpeed = ballView->getSpeedVector();
  
  PointUE agentPos = agentView->getPosition();
  VectorUE relBallVector = ballView->getRelativeVector();

  Point goalCenter = facts->getOurGoal().getPosition();

  Plan plan;
  Step step;
  
  static Int lastCatch = 0;
  lastCatch--;

  if (gameView->getPlayMode() == PM_OUR_GOALIE_GOT_BALL ||
      gameView->getPlayMode() == PM_OUR_GOAL_KICK ||
      (gameView->getPlayMode() == PM_OUR_FREE_KICK &&
       ballView->isBallKickable())) {
    AngleDeg turnAngle = 0;
    if (agentPos.isKnown() && agentView->getBodyDirection().isKnown()) 
      turnAngle = (agentPos.getDirectionTo(facts->getMarker(FLAG_C).getPosition()) -
		   agentView->getBodyDirection()).getDeg();
    else
      turnAngle = 45;
    
    if (abs(turnAngle) > 5) {
      TurnCommand* tc = new TurnCommand(turnAngle);
      RS_LOG ( LA_SKILL, "GoaliePositioning - turn after catch: " << turnAngle);
      plan.addStep(step.addCommand(tc));
      delete tc;
    }
    else {
      AngleDeg kickdir = (agentPos.getDirectionTo(facts->getMarker(FLAG_C).getPosition()) -
			  agentView->getBodyDirection()).getDeg();
      KickCommand* kc = new KickCommand(100, kickdir);
      RS_LOG ( LA_SKILL, "GoaliePositioning - kick ball towards center: " << kickdir);
      plan.addStep(step.addCommand(kc));
      delete kc;
    }
    return plan;
  }
  // Check if ball is catchable, if so catch it
  if (ballView->isBallCatchable() && lastCatch<0) {
    AngleDeg catchAngle = (relBallVector.getAngle() - agentView->getBodyDirection()).getDeg();
    lastCatch = facts->SP_catch_ban_cycle;
    CatchCommand* cc = new CatchCommand(catchAngle);
    RS_LOG ( LA_SKILL, "GoaliePositioning - catch: " << catchAngle);        
    plan.addStep(step.addCommand(cc));
    delete cc;
    return plan;
  }

  // If we missed a catch and the ball is kickable, kick it.
  // Fix this to clear the ball insted.
  if(ballView->isBallKickable() && lastCatch>=0) {
    AngleDeg kickdir = 0;
    if (agentPos.isKnown() && agentView->getBodyDirection().isKnown())
      kickdir = (agentPos.getDirectionTo(facts->getMarker(FLAG_C).getPosition()) -
		 agentView->getBodyDirection()).getDeg();
    KickCommand* kc = new KickCommand(100, kickdir);
    RS_LOG ( LA_SKILL, "GoaliePositioning - panickick: " << kickdir);        
    plan.addStep(step.addCommand(kc));
    delete kc;
    return plan;
  }
  
  //Check how close the opposition is to the ball
  FloatUE closeOpDist
    (playerView->getOpponentClosestToBall().getPosition().getDistanceTo(ballPos));
  
  //Predict where to intercept ball
  PointUE predBallPos = ballPos;
    
  if (ballSpeed.isKnown() && relBallVector.isKnown() &&
      closeOpDist.isKnown()) {
    //do a ball prediction to see if we want to rush out
    PointUE tmpPredBall = ballPos;
    VectorUE tmpPredBallSpeed = ballSpeed;
    Int maxPred=15;
    if(closeOpDist.isKnown() && (maxPred > closeOpDist.getValue()*3.0))
      maxPred=(int)(closeOpDist.getValue()*3);
    for(int a=0; a<maxPred; a++) {      
      RS_LOG (LA_SKILL, "  predBallPos: " << tmpPredBall.getPoint());
      RS_LOG (LA_SKILL, "     distance: " << agentPos.getDistanceTo(tmpPredBall).getValue());
      tmpPredBall += tmpPredBallSpeed;
      tmpPredBallSpeed *= facts->SP_ball_decay;
      if(goalCenter.getDistanceTo(tmpPredBall.getPoint()) < 15) {
	if (agentPos.getDistanceTo(tmpPredBall) < agentPos.getDistanceTo(predBallPos))
	  predBallPos = tmpPredBall;
      }
    }
  }
  
  //The positioning is done here
  
  //Calculate the vector between the own goal and the ball
  VectorUE ballGoalVec;
  
  FloatUE ballDistance;
  if (predBallPos.isKnown()) {
    ballGoalVec = VectorUE(goalCenter, predBallPos);
    ballDistance = ballGoalVec.getLength();
    
    //place the goalie on a 1/0.15-radius-circle from the center of the goal
    ballGoalVec/=(ballDistance*0.15);
  }
  
  Point moveToPos;
  FloatUE predBallDist = predBallPos.getDistanceTo(goalCenter);
  Float maxDist = 1.0;
  
  if (predBallPos.isKnown() && (predBallDist < 15)) {
    RS_LOG ( LA_SKILL, "GoaliePositioning - run out to comming position.");      
    maxDist=1.0;
    moveToPos=predBallPos.getPoint();
  }
  else if (ballGoalVec.isKnown()) {
    //Place the goalie on circle    
    RS_LOG ( LA_SKILL, "GoaliePositioning - place goalie on circle.");      
    maxDist=0.8;
    moveToPos = (goalCenter+ballGoalVec).getPoint();
  }
  else if (agentPos.isKnown()) {
    // The agent is positioned the rigth way.
    RS_LOG ( LA_SKILL, "GoaliePositioning - don't run.");      
    moveToPos = agentPos.getPoint();
  }
  else {
    RS_LOG ( LA_SKILL, "GoaliePositioning - position unknown");      
    moveToPos = Point(0,0);
  }
  
  //use more dashpower if ball is close
  Float dashPower;
  if (ballDistance.isKnown())
    dashPower = 140-ballDistance.getValue()*2;
  else
    dashPower = 100;

  if(dashPower>100)
    dashPower=100;
  else if (dashPower<5)
    dashPower=5;
  
  MoveToPosition moveToPosition(moveToPos, (int)dashPower, maxDist, 5);
  FindBall findBall;
  // Move to the calculated position.
  if (moveToPosition.applicable()) {
    Plan plan = moveToPosition.generatePlan();
    Step step = plan.getNextStep();
    AngleDeg turnAngle(moveToPosition.getTurn());
    
    // If ball is known look at it (if possible)
    AngleDegUE neckTurn(relBallVector.getAngle() - turnAngle - 
			  agentView->getBodyDirection());
    if (neckTurn > 90) neckTurn = 90;
    if (neckTurn < -90) neckTurn = -90;
    TurnNeckCommand* neckcmd = new TurnNeckCommand(neckTurn.getDeg());
    step.addCommand(neckcmd);
    delete neckcmd;
    RS_LOG ( LA_SKILL, "GoaliePositioning - MoveToPosition: " << moveToPos);      
    return Plan(step);
  }
  // Look for ball.
  else if (findBall.applicable()) {
    RS_LOG ( LA_SKILL, "GoaliePositioning - FindBall");      
    return findBall.generatePlan();
  }
  // Look at ball.
  else if (agentView->getBodyDirection().isKnown() &&
	   agentView->getNeckDirection().isKnown() &&
	   ballPos.isKnown()) {
    // Align neck and body.
    TurnNeckCommand* neckcmd = 
      new TurnNeckCommand((agentView->getBodyDirection() - 
			   agentView->getNeckDirection()).getDeg());
    step.addCommand(neckcmd);
    delete neckcmd;
    TurnCommand* turncmd =
      new TurnCommand((agentPos.getDirectionTo(ballPos) - 
		       agentView->getBodyDirection()).getDeg());
    plan.addStep(step.addCommand(turncmd));
    RS_LOG ( LA_SKILL, "GoaliePositioning - LookAtBall: "
	     << (agentPos.getDirectionTo(ballPos) - agentView->getBodyDirection()).getDeg());
    delete turncmd;
    return plan;
  }
  // Bodydirection or NeckDirection unknown, do turn to see something.
  else {
    RS_LOG ( LA_SKILL, "GoaliePositioning - Turn to get bodydir");      
    TurnCommand* turncmd = new TurnCommand(45);
    plan.addStep(step.addCommand(turncmd));
    delete turncmd;
    return plan;
  }
}


FuzzyBool GoaliePositioning::persistent() {
  // Never persistent.
  return FuzzyBool(0.0);
}

FuzzyBool GoaliePositioning::applicable() {
  // Applicable if goalie
  if(facts->CP_goalie) {
//      RS_LOG ( LA_SKILL, "GoaliePositioning - applicable");    
    return FuzzyBool(1.0);  
  }
  return FuzzyBool(0.0);
}

FuzzyBool GoaliePositioning::succeed() {
  // Always succedes.
  return FuzzyBool(1.0);
}

FuzzyBool GoaliePositioning::failed() {
  // Never fails.
  return FuzzyBool(0.0);
}


RS_END_NAMESPACE



