/* -*- Mode: C++ -*-
 * $Id: GotoBall.cc,v 1.10 2002/09/30 14:51:47 aip4 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GotoBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.10 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: aip4 $ $Date: 2002/09/30 14:51:47 $
 *
 *
 * HISTORY
 * $Log: GotoBall.cc,v $
 * Revision 1.10  2002/09/30 14:51:47  aip4
 * Committed first versions of Mark skill
 *
 * Revision 1.9  2002/09/04 13:27:48  s02davro
 * Updated GotoBall to handle new world model
 *
 * Revision 1.8  2001/09/29 18:55:20  frehe
 * Removed TRUE, FALSE, and UNKNOWN.
 *
 * Revision 1.7  2001/09/26 11:41:25  frehe
 * Update after move of TRUE, FALSE and UNKNOWN to inside FuzzyBool.
 *
 * Revision 1.6  2000/10/09 22:13:01  chrro564
 * Updated MoveToPositioin skill.
 *
 * Revision 1.5  2000/10/09 11:59:48  olalu526
 * Fixed applicable references (so that it points to the one that it inherits from.
 *
 * Revision 1.4  2000/10/09 11:07:47  olalu526
 * Made so that GotoBall inherits from MoveToPositionFacingBall.
 *
 * Revision 1.3  2000/10/01 13:05:54  olalu526
 * Updated FacingDirection, fixing bugs and stuff.
 *
 * Revision 1.2  2000/09/13 01:38:59  frehe
 * Updated the skills to use getView instead of static_cast<...>.
 *
 * Revision 1.1  2000/09/05 21:12:01  frehe
 * #include rs_log.h
 *
 */

/**
 * @file GotoBall.cc
 * Implementation file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.10 $
 * $Author: aip4 $
 * $Date: 2002/09/30 14:51:47 $
 */

#include "MoveToPosition.h"
#include "GotoBall.h"
#include "BallView.h"
#include "WorldFacts.h"
#include "view_constants.h"
#include "TurnCommand.h"
#include "DashCommand.h"


RS_BEGIN_NAMESPACE


GotoBall::GotoBall(const Int def_dash,
		   const AngleDeg def_minTurnAngle)
  : facts(WorldFacts::instance()),
    ballView(NULL) {
  getView(ballView, BALL_VIEW_ID);
  setMaxDistance(facts->SP_kickable_margin);
  setMinTurnAngle(def_minTurnAngle);
  setDash(def_dash);
} 

GotoBall::~GotoBall() {
  viewManager->releaseView(BALL_VIEW_ID);
}

Plan GotoBall::generatePlan() {
  if (ballView->getPosition(NEXT).isKnown())
    setPoint(ballView->getPosition(NEXT).getPoint());
  return MoveToPosition::generatePlan();
}

FuzzyBool GotoBall::persistent() {
  return FuzzyBool(0.3);
}

FuzzyBool GotoBall::applicable() {
  if (ballView->getPosition(NEXT).isKnown()) {
    setPoint(ballView->getPosition(NEXT).getPoint());
    return MoveToPosition::applicable();
  }
  return FuzzyBool(0.0);
}

RS_END_NAMESPACE





