/* -*- Mode: C++ -*-
 * $Id: GotoBallClosest.cc,v 1.1 2002/10/03 09:53:16 aip11 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GotoBallClosest.cc
 * Unitname: Framework/skill
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: aip11 $ $Date: 2002/10/03 09:53:16 $
 *
 *
 * HISTORY
 */

/**
 * @file GotoBallClosest.cc
 * Implementation file for the skill class GotoBallClosest.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: aip11 $
 * $Date: 2002/10/03 09:53:16 $
 */

#include "MoveToPosition.h"
#include "GotoBallClosest.h"
#include "BallView.h"
#include "AgentView.h"
#include "PlayerView.h"
#include "WorldFacts.h"
#include "view_constants.h"
#include "TurnCommand.h"
#include "DashCommand.h"
//#include "Scan.h"

RS_BEGIN_NAMESPACE


GotoBallClosest::GotoBallClosest(const Int def_dash,
		   const AngleDeg def_minTurnAngle)
  : facts(WorldFacts::instance()),
    ballView(NULL),
    agentView(NULL),
    playerView(NULL) {
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
  setMaxDistance(facts->SP_kickable_margin);
  setMinTurnAngle(def_minTurnAngle);
  setDash(def_dash);
} 

GotoBallClosest::~GotoBallClosest() {
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
}

Plan GotoBallClosest::generatePlan() {
  
  if (ballView->getSpeedVector().isKnown())
    setPoint(ballView->getPosition().getPoint() + 
	     ballView->getSpeedVector().getVector());
  else
    setPoint(ballView->getPosition().getPoint());
  
  return MoveToPosition::generatePlan();
  
}
FuzzyBool GotoBallClosest::persistent() {
  return FuzzyBool(0.0);
}

FuzzyBool GotoBallClosest::applicable() {

  if (ballView->getPosition().isKnown()) {
    //RS_LOG (LA_SKILL,"I know where ball is");
    
    PlayerObject closestPlayer = playerView->getTeammateClosestToBall();
    
    if (agentView->getPosition().isKnown() && 
	closestPlayer.getPosition().isKnown()) {
      
   
      if (Vector(agentView->getPosition().getPoint(), 
		 ballView->getPosition().getPoint()).getLength() <
	  Vector(closestPlayer.getPosition().getPoint(), 
		 ballView->getPosition().getPoint()).getLength()) {
	
	//RS_LOG (LA_SKILL,"I'm closest");
	setPoint(ballView->getPosition().getPoint());
 	return MoveToPosition::applicable();
      } else {
	//RS_LOG (LA_SKILL,"Someone else is closest");
	return FuzzyBool(0.0);
      } 
    } else {
      //RS_LOG (LA_SKILL,"I'm all alone");
       setPoint(ballView->getPosition().getPoint());
       return MoveToPosition::applicable();
     }
  }
  //RS_LOG (LA_SKILL,"Where is the ball?");
  return FuzzyBool(0.0);
}

RS_END_NAMESPACE









