/* -*- Mode: C++ -*-
 * $Id: GotoBallClosest.h,v 1.1 2002/10/03 09:53:16 aip11 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GotoBallClosest.h
 * Unitname: Framework/skill
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: aip11 $ $Date: 2002/10/03 09:53:16 $
 *
 *
 * HISTORY
 * $Log: GotoBallClosest.h,v $
 * Revision 1.1  2002/10/03 09:53:16  aip11
 * Added GotoBallClosest which moves the teammate that thinks he is closest to the ball towards it.
 *
 *
 */

/**
 * @file GotoBallClosest.h
 * Include file for the skill class GotoBallClosest.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: aip11 $
 * $Date: 2002/10/03 09:53:16 $
 */


#ifndef GOTOBALL_H
#define GOTOBALL_H

#include "MoveToPosition.h"
#include "PlayerView.h"
#include "AgentView.h"
#include "WorldFacts.h"

RS_BEGIN_NAMESPACE

class BallView;

/**
 * @class GotoBallClosest GotoBallClosest.h
 * This class encapsulates a go to the ball skill.
 *
 * <!--
 * @pre The preconditions are that the player thinks he is closest to the ball.
 * @post The postconditions player moves towards the ball.
 * @invariant The class invariant is...
 *
 * @todo This needs to be done...
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: aip11 $
 * $Date: 2002/10/03 09:53:16 $
 */
class GotoBallClosest : public MoveToPosition {
public:
  GotoBallClosest(const Int def_dash = 50,
	   const AngleDeg def_minTurnAngle = AngleDeg(5.0));
  virtual ~GotoBallClosest();
  virtual Plan generatePlan();
  virtual FuzzyBool persistent();
  virtual FuzzyBool applicable();
protected:  
  const WorldFacts* facts;
  BallView* ballView;
  AgentView* agentView;
  PlayerView* playerView;
};


RS_END_NAMESPACE


#endif // !GOTOBALL_H
