/* -*- Mode: C++ -*-
 * $Id: KickBallToSpeedVector.cc,v 1.12 2002/09/04 13:28:15 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: KickBallToSpeedVector.cc
 * Unitname: Framework/skill
 * $Revision: 1.12 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2002/09/04 13:28:15 $
 *
 *
 * HISTORY
 *
 * $Log: KickBallToSpeedVector.cc,v $
 * Revision 1.12  2002/09/04 13:28:15  s02davro
 * Updated KickBallToSpeedVector, removed many things
 *
 * Revision 1.11  2001/09/29 18:55:21  frehe
 * Removed TRUE, FALSE, and UNKNOWN.
 *
 * Revision 1.10  2001/09/26 11:41:26  frehe
 * Update after move of TRUE, FALSE and UNKNOWN to inside FuzzyBool.
 *
 * Revision 1.9  2001/08/23 00:31:59  frehe
 * Removed dependencies.
 *
 * Revision 1.8  2000/11/08 19:32:07  mikbr212
 * Fixed mem-leak in generatePlan
 *
 * Revision 1.7  2000/11/07 18:10:43  mikbr212
 * changed persistent
 *
 * Revision 1.6  2000/10/30 18:32:45  mikbr212
 * added comments
 *
 */

/**
 * @file KickBallToSpeedVector.cc
 * Implementation file for the skill class KickBallToSpeedVector.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.12 $
 * $Author: s02davro $
 * $Date: 2002/09/04 13:28:15 $
 */

#include "KickBallToSpeedVector.h"
#include "KickBallView.h"
#include "BallView.h"
#include "KickCommand.h"

RS_BEGIN_NAMESPACE


/**
 * Constructor.
 */
KickBallToSpeedVector::KickBallToSpeedVector(const Vector& vec)
  : kickBallView(NULL) {
  getView(kickBallView, KICKBALL_VIEW_ID);
  getView(ballView, BALL_VIEW_ID);
} 

/**
 * Destructor.
 */
KickBallToSpeedVector::~KickBallToSpeedVector() {
  viewManager->releaseView(KICKBALL_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
}

/**
 * Sets vector.
 */
void KickBallToSpeedVector::setVector(const Vector& def_kickVector) {
  kickVector = def_kickVector;
}

/**
 * Retrieves vector.
 */
Vector KickBallToSpeedVector::getVector() const {
  return kickVector;
}

/**
 * Generates a plan.
 */
Plan KickBallToSpeedVector::generatePlan() {
  static Command* cmd = NULL;
  Step step;

  VectorUE vectorForKick = kickBallView->getKickVector(kickVector);

  if(ballView->isBallKickable() && vectorForKick.isKnown()) {
    cmd = new KickCommand((vectorForKick.getLength()/
			   kickBallView->kickPowerRate()).getValue(),
			  vectorForKick.getAngle().getDeg());
    step.addCommand(cmd);
    delete cmd;
  }
  return Plan(step);
}

/**
 * Checks if this skill should be called again (this cycle).
 */
FuzzyBool KickBallToSpeedVector::persistent() {
  return FuzzyBool(0.0); // This is a one-step skill
}

/**
 * Checks if the preconds are true.
 */
FuzzyBool KickBallToSpeedVector::applicable() {
  return kickBallView->canShootToSpeedVector(kickVector);
}

RS_END_NAMESPACE
