/* -*- Mode: C++ -*-
 * $Id: KickBallToSpeedVector.h,v 1.7 2002/09/04 13:28:14 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnBall.h
 * Unitname: Framework/skill
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2002/09/04 13:28:14 $
 *
 *
 * HISTORY
 * $Log: KickBallToSpeedVector.h,v $
 * Revision 1.7  2002/09/04 13:28:14  s02davro
 * Updated KickBallToSpeedVector, removed many things
 *
 * Revision 1.6  2001/09/01 23:51:46  frehe
 * Updated the skill with the new abstract skill class and the new version of getValue and getAngle.
 *
 * Revision 1.5  2001/08/23 00:31:59  frehe
 * Removed dependencies.
 *
 * Revision 1.4  2000/10/09 18:48:09  mikbr212
 * Added comments
 *
 * Revision 1.3  2000/10/07 14:26:55  mikbr212
 * *** empty log message ***
 *
 * Revision 1.2  2000/09/27 14:29:59  mikbr212
 * fixed ballspeed bug
 *
 * Revision 1.1  2000/09/24 19:30:19  andbo928
 * Various skills.
 *
 * Revision 1.1  2000/09/05 21:12:02  frehe
 * #include rs_log.h
 *
 */

#ifndef KICKBALLTOSPEEDVECTOR_H
#define KICKBALLTOSPEEDVECTOR_H


#include "Skill.h"
#include "view_constants.h"

RS_BEGIN_NAMESPACE

class BallView;
class KickBallView;

/**
 * @class KickBallToSpeedVector KickBallToSpeedVector.h
 * This class encapsulates a kicking skill.
 * 
 * @pre The preconditions for the skill are:
 * 1) ball in kickable_area
 * 2) no obstacle on straight line from current ball position to next ball
 *    position 
 * @post The postconditions for the skill are...
 * 
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: s02davro $
 * $Date: 2002/09/04 13:28:14 $
 */

class KickBallToSpeedVector : public Skill {
public:
  KickBallToSpeedVector(const Vector& def_kickVector = Vector(0,1));
  virtual ~KickBallToSpeedVector();
  
  virtual void setVector(const Vector& def_kickVector);
  virtual Vector getVector() const;
  
  virtual Plan generatePlan();
  virtual FuzzyBool persistent();  
  virtual FuzzyBool applicable();
  
  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed() { return FuzzyBool(0.5); };

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed() { return FuzzyBool(0.5); };
protected:
  KickBallView* kickBallView;
  BallView* ballView;
  Vector kickVector;  
};


RS_END_NAMESPACE


#endif // !KICKBALLTOSPEEDVECTOR_H


