/* -*- Mode: C++ -*-
 * $Id: KickToControlBall.cc,v 1.3 2002/10/14 15:15:26 aip8 Exp $Date 2002/09/27 00:00:00 aip8 $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is used in the framework provided by RoboSoc created by
 * Fredrik Heintz <frehe@ida.liu.se>.
 *
 * IDENTIFICATION
 *
 * Filename: KickToControlBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.3 $
 * Created by: aip8 $Date
 * Last modified by $Author: aip8 $ $Date 2002/09/27 00:00:00 $
 *
 *
 * HISTORY
 *
 * $Log: KickToControlBall.cc,v $
 * Revision 1.3  2002/10/14 15:15:26  aip8
 * *** empty log message ***
 *
 * Revision 1.2  2002/09/29 22:27:30  aip8
 * MoveWithBallToPosition2.cc/.h are more primitive but faster versions of MoveWithBallToPosition.cc/.h
 * The other files are implementations of skills.
 *
 *
 * Revision 0.4  2002/09/27 19:00:00  aip8
 * It should work at least in the simplest cases.
 * Added the KickToControlBall::normalize private method.
 *
 * Revision 0.1  2002/09/27 00:00:00  aip8
 * It does something not well specified...
 *
 * Revision 0.0  2002/09/18 17:55:02  aip8
 * It compiles, that's all it does.
 *
 */

/**
 * @file KickToControlBall.cc
 * Implementation file for the skill class KickToControlBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: aip8 $
 * $Date: 2002/10/14 15:15:26 $
 */

#include "AgentView.h"
#include "BallView.h"
//#include "GameView.h"
//#include "PlayerView.h"
#include "KickCommand.h"
#include "KickToControlBall.h"
#include "DashCommand.h"
#include "view_constants.h"
#include "MathVectorUE.h"
#include "TurnCommand.h"
#include "WorldFacts.h"

RS_BEGIN_NAMESPACE

// --- CONSTRUCTORS ---

KickToControlBall::KickToControlBall(Float a = 360.0, Float v = -1.0)
  : Skill(), ballView(NULL), agentView(NULL)//, gameView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  //getView(gameView, GAME_VIEW_ID);

  if (a != 360.0)
    if ( agentView->getBodyDirection().isKnown() )
      finalDir = AngleDeg(agentView->getBodyDirection().getValue());
    else
      finalDir = AngleDeg(0.0);
  else
    finalDir = AngleDeg(a);

  if (v < 0.0)
    if ( agentView->getSpeed().isKnown() )
      finalSpeed = agentView->getSpeed().getValue();
    else
      finalSpeed = 0.0;
  else
    finalSpeed = v;

  facts = WorldFacts::instance();

}


// --- DESTRUCTOR ---

KickToControlBall::~KickToControlBall()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  //viewManager->releaseView(PLAYER_VIEW_ID);
}


// --- ACCESSORS ---

void KickToControlBall::setFinalDir(AngleDeg fd)
{
  finalDir = normalize(fd);
  RS_LOG(LA_SKILL, "$$ requested dir: " << finalDir.getDeg());
}

AngleDeg KickToControlBall::getFinalDir()
{
  return finalDir;
}

void KickToControlBall::setFinalSpeed(Float fs)
{
  finalSpeed = fs;
  RS_LOG(LA_SKILL, "$$ requested speed: " << finalSpeed);
}

Float KickToControlBall::getFinalSpeed()
{
  return finalSpeed;
}

// --- PUBLIC METHODS ---

Plan KickToControlBall::generatePlan()
{
  /* This is a versor */
  //  MathVectorUE agentAbsDir = MathVectorUE( agentView->getBodyDirection(), FloatUE(1.0) ); 
  agentAbsDir = MathVectorUE( agentView->getBodyDirection(), FloatUE(1.0) ); 

  // ?? should cleanup all this ugly stuff
  AngleDeg tt;

  PointUE agentNextAbsPos = agentView->getPosition(NEXT);
  PointUE ballCurrAbsPos = ballView->getPosition();
  VectorUE ballSpeed = ballView->getSpeedVector();
  VectorUE agentSpeed = agentView->getSpeedVector();
  Float32 dashingPower = requiredDashingPower();

  Float p_decay = facts->SP_player_decay;
  Float b_decay = facts->SP_ball_decay;
  Float moment;

  Command * cmd;
  Step step0, step1, step2, step3, step4;
  Plan p;

  /* unknowns */
  AngleDegUE kickingDir;
  FloatUE kickingPower;  



  if ( agentAbsDir.isUnknown() || agentNextAbsPos.isUnknown() || ballCurrAbsPos.isUnknown() || ballSpeed.isUnknown() 
       || agentSpeed.isUnknown() )
    return Plan();


  RS_LOG(LA_SKILL, "$$ requested speed: " << finalSpeed);
  RS_LOG(LA_SKILL, "------ FACTS:" << std::endl <<
	 "## agentPosition(x, y): " << "(" << agentView->getPosition(NOW).getX().getValue() << ", " << agentView->getPosition(NOW).getY().getValue() << ")" << std::endl <<
	 "## agentNextAbsPos(x, y): " << "(" << agentNextAbsPos.getX().getValue() << ", " << agentNextAbsPos.getY().getValue() << ")" << std::endl <<
	 "## ballCurrAbsPos(x, y): " << "(" << ballCurrAbsPos.getX().getValue() << ", " << agentNextAbsPos.getY().getValue() << ")" << std::endl <<
	 "## ballNextAbsPos(x, y): " << "(" << ballView->getPosition(NEXT).getX().getValue() << ", " << ballView->getPosition(NEXT).getY().getValue() << ")" << std::endl <<
	  "## ballSpeed(lenght, angle): (" << ballSpeed.getLength().getValue() << ", " << ballSpeed.getAngle().getDeg() << "')" << std::endl <<
	 "## agentSpeed(lenght, angle): (" << agentSpeed.getLength().getValue() << ", " << agentSpeed.getAngle().getDeg() << "')" <<std::endl <<
	 "## agentAbsDir: " << agentView->getBodyDirection().getDeg() << "'"

	 );


  VectorUE t2VAgent = sqr(p_decay) * agentSpeed;
  RS_LOG(LA_SKILL, "------ CALCULATIONS" << std::endl << "## t2VAgent(l,a): (" << t2VAgent.getLength().getValue() <<
	 ", " << t2VAgent.getAngle().getDeg() << "')");


  // angle between the t+2 speed vector and the required dashing vector
  VectorUE acceleration = ( t2VAgent + VectorUE(finalDir, finalSpeed).rotate(AngleDegUE(180.0)) ).rotate(AngleDegUE(180.0));
  // in radians
  AngleRad dashingDir = AngleRad( MathVectorUE( acceleration ).angleBetween(t2VAgent).getValue() );
  RS_LOG(LA_SKILL, "%%% acceleration(l, a), dashingDir: (" << acceleration.getLength().getValue() << ", " <<
	 acceleration.getAngle().getDeg() << ") - " << dashingDir.getDeg()); 
  
  // some values...
  Float t0VxAgent = agentSpeed.getLength().getValue() * cos(agentSpeed.getAngle().getRad());
  Float t0VyAgent = agentSpeed.getLength().getValue() * sin(agentSpeed.getAngle().getRad());

  Float t0VxBall = ballSpeed.getLength().getValue() * cos( ballSpeed.getAngle().getRad() );
  Float t0VyBall = ballSpeed.getLength().getValue() * sin( ballSpeed.getAngle().getRad() );

  /* STEP t=0: Control the ball and kick it to position we'll be in the 4th cycle */

  // ?? I'm not checking if the next absolute position of the agent is illegal
  // ?? i.e. outside the field, offside, ...


//    Float t4DeltaPxAgent = (1 + sqr(p_decay) * ( 2 + p_decay)) *
//                      agentSpeed.getLength().getValue() *
//                      cos(agentSpeed.getAngle().getValue()) +
//                      2 * power * cos(dashingDir) ;
//    Float t4DeltaPyAgent = (1 + sqr(p_decay) * ( 2 + p_decay)) *
//                      agentSpeed.getLength().getValue() *
//                      sin(agentSpeed.getAngle().getValue()) +
//                      2 * power * sin(dashingDir) ;

  /*
    The following are the components of the distance vector from point at time 0 to t+4 .
    To obtain the absolute position (t4PxAgent, t4PyAgent) simply add (t0PxAgent, t0PyAgent) to it.
  */
  Float t4DeltaPxAgent = agentView->getPosition(NOW).getX().getValue()  +  t0VxAgent * (1 + p_decay * (1 + p_decay + sqr(p_decay))) +
                         2 * dashingPower * cos(dashingDir.getRad());
  Float t4DeltaPyAgent = agentView->getPosition(NOW).getY().getValue()  +   t0VyAgent * (1 + p_decay * (1 + p_decay + sqr(p_decay))) +
                         2 * dashingPower * sin(dashingDir.getRad());
  
  RS_LOG(LA_SKILL, "## (t4DeltaPxAgent, t4DeltaPyAgent): (" << t4DeltaPxAgent << ", " << t4DeltaPyAgent << ")");

  /*
    In its final position, the ball should not overlap with the agent
    NOTE: the simulation model is discrete, i.e. if we were in the real world, it could have been possible to
          have the ball "passing through" the agent and having it anyway in the right position.
  */
  updWithObjSizes(t4DeltaPxAgent, t4DeltaPyAgent);
  RS_LOG(LA_SKILL, "## UPDATED (t4DeltaPxAgent, t4DeltaPyAgent): (" << t4DeltaPxAgent << ", " << t4DeltaPyAgent << ")");

  /* 
     these values should be used for the Kick command in order to find the ball
     (during next cycle [and ?? ignoring opponets]) in the best position for the agent.
  
     If we get Atan2(0,0) we simply need the angle between body direction and the
     opposite of the ball's speed vector.

     ?? COSTRAINS: ballNextAbsDir != 0+k*PI && ballNextAbsDir != PI/2 + k*PI
  */
  Float dy = t4DeltaPyAgent - t0VyBall;
  Float dx = t4DeltaPxAgent - t0VxBall;
  AngleRad ballNextAbsDir = AngleRad( ( dx || dy ) ? 
                                      atan2( dy, dx ) :
				      (agentAbsDir.angleBetween( ballSpeed.rotate(AngleDegUE(180.0))) ).getValue()
				    );

  RS_LOG(LA_SKILL, "## ballNextAbsDir: " << ballNextAbsDir.getDeg() << "'");   

  tt = AngleDeg(ballNextAbsDir.getDeg());
  kickingDir = normalize(tt - agentAbsDir.getAngle().getAngle());
  // ?? cos(ballNextAbsDir) == 0

  kickingPower = dx * ( 1.0 / (ekpCoefficient() * cos( ballNextAbsDir.getRad() )).getValue() );
  FloatUE XX = dy * ( 1.0 / (ekpCoefficient() * sin( ballNextAbsDir.getRad() )).getValue() );
  RS_LOG(LA_SKILL, "kickingPower(x), kickingPower(y) = " << kickingPower.getValue() << ", " << XX.getValue());

  RS_LOG(LA_SKILL, "STEP0: kickingDir: " << kickingDir.getDeg() << " kickingPower: " << kickingPower.getValue());


  kickingPower = FloatUE(20.0);


  // the following code has to be always executed (~ FAST)
  if (kickingPower < facts->SP_minpower) {
    kickingPower = facts->SP_minpower;
    // figure out next position and cope with it
  } else if (kickingPower > facts->SP_maxpower) {
    kickingPower = facts->SP_maxpower;
    // figure out next position and cope with it
  }

  cmd = new KickCommand(kickingPower.getValue(), kickingDir.getValue());
  step0.addCommand(cmd);
  delete cmd;
  p.addStep(step0);

  /* if we want just to send the ball in front of us but with different speed and direction
     compared to ours...

if (how == FAST) 
  return p; 

  */

  /* STEP t=1: Turn the agent */



  // Float t3Vx = finalSpeed * cos(finalDir);
  // Float t3Vy = finalSpeed * sin(finalDir);
  // Float t2Vx = sqr(facts->SP_player_decay) * t0Vx;
  // Float t2Vy = sqr(facts->SP_player_decay) * t0Vy;
  // NOTE: dashingDir != finalDir
   // RS_LOG(LA_SKILL,  << "<KickToControlBall> 229/287";

  moment = requiredMoment(tt);
  cmd = new TurnCommand(KickToControlBall::normalize(AngleDeg(moment)));
  step1.addCommand(cmd);
  delete cmd;
  p.addStep(step1);
  RS_LOG(LA_SKILL, "STEP1: (Turn to) dashingDir: " << moment << "'");

  /* STEP t=2: dash in order to get desired final speed. */
  RS_LOG(LA_SKILL, "STEP2: requiredDashingPower: " << dashingPower);
  cmd = new DashCommand(dashingPower);
  step2.addCommand(cmd);
  delete cmd;
  p.addStep(step2);

  /* STEP t=3: turn again so that the agent will be facing on the requested direction. */
  moment = requiredMoment(finalDir);
  cmd = new TurnCommand(KickToControlBall::normalize(AngleDeg(moment)));
  step3.addCommand(cmd);
  delete cmd;
  p.addStep(step3);
  RS_LOG(LA_SKILL,  "STEP3: (Turn again to) finalDir: " << finalDir.getDeg() << "'");

  /* STEP t=4: Make the ball have (~) same speed and direction of the agent.  */
 
  // the ball now is at t4PAgent (updated values) with speed b_decay**4 * ( t0VBall + kickingPower * ekpCoefficient() )
  Float t4VxBall = pow(b_decay, 4) * ( t0VxBall + kickingPower.getValue() * ekpCoefficient().getValue());
  Float t4VyBall = pow(b_decay, 4) * ( t0VyBall + kickingPower.getValue() * ekpCoefficient().getValue());

  //  Float t5PxAgent = t4DeltaPxAgent + sqr(p_decay) * ( sqr(p_decay) * t0VxBall + acceleration.getLength().getValue() );
  //  Float t5PyAgent = t4DeltaPyAgent + sqr(p_decay) * ( sqr(p_decay) * t0VyBall + acceleration.getLength().getValue() );
  Float t5PxAgent = sqr(p_decay) * ( sqr(p_decay) * t0VxBall + acceleration.getLength().getValue() );
  Float t5PyAgent = sqr(p_decay) * ( sqr(p_decay) * t0VyBall + acceleration.getLength().getValue() );
  // ?? COSTRAINS: ballNextAbsDir != 0+k*PI && ballNextAbsDir != PI/2 + k*PI
  dy = t5PyAgent - t4VyBall;
  dx = t5PxAgent - t4VxBall;
  ballNextAbsDir = AngleRad( ( dx || dy ) ? 
			     atan2( dy, dx ) :
			     (agentAbsDir.angleBetween( ballSpeed.rotate(AngleDegUE(180))) ).getValue()
			   );

  tt = AngleDeg(ballNextAbsDir.getDeg());
  kickingDir = KickToControlBall::normalize(tt - agentAbsDir.getAngle().getAngle());
  // ?? cos(ballNextAbsDir) == 0
  kickingPower = dx * ( 1.0 / (ekpCoefficient() * cos( ballNextAbsDir.getRad() )).getValue() );

  // the following code has to be always executed (~ FAST)
  if (kickingPower < facts->SP_minpower) {
    kickingPower = facts->SP_minpower;
    // figure out next position and cope with it
  } else if (kickingPower > facts->SP_maxpower) {
    kickingPower = facts->SP_maxpower;
    // figure out next position and cope with it
  }


  kickingPower = FloatUE(20.0);


  RS_LOG(LA_SKILL, "STEP4: kickingDir: " << kickingDir.getDeg() << " kickingPower: " << kickingPower.getValue());
  cmd = new KickCommand(kickingPower.getValue(), kickingDir.getValue());
  step4.addCommand(cmd);
  delete cmd;
  p.addStep(step4);
   // RS_LOG(LA_SKILL,  << "<KickToControlBall> 285/287";

  return p;
  
}


FuzzyBool KickToControlBall::persistent()
{
  // Never persistent.
  return FuzzyBool(1.0);
}


FuzzyBool KickToControlBall::applicable()
{
  // ?? i should use a predicate()
  //if (ballView->isBallKickable())
  return ballView->isBallKickable();
}

FuzzyBool KickToControlBall::succeed()
{
  // Always succedes.
  return FuzzyBool(1.0);
}

FuzzyBool KickToControlBall::failed()
{
  // Never fails.
  return FuzzyBool(0.0);
}








// ?? ignoring WithError
FloatUE KickToControlBall::ekpCoefficient()
{
  // must be the distance between the borders!!
  FloatUE ball_dist = ballView->getRelativeDistance() - facts->SP_player_size - facts->SP_ball_size;
  // must be the absolute direction difference
  AngleDegUE rel_dir = ballView->getRelativeDirection();

  if (ball_dist.isUnknown() || rel_dir.isUnknown())
    return FloatUE();
  Float rd = rel_dir.getDeg();
  Float bd = ball_dist.getValue();

  FloatUE u = facts->SP_kick_power_rate * ( 1.0 - 
				       0.25 * rd * (1 / 180) -
				       0.25 * bd * (1 / facts->SP_kickable_margin)
				     );
  return u;
}


AngleDeg KickToControlBall::normalize(AngleDeg alpha)
{
  Float a = alpha.getDeg();
  // RS_LOG(LA_SKILL,  << "<KickToControlBall> normalize: beginning";
  while (a <= 180.0)
    a += 360.0;
  while (a > 180)
    a -= 360.0;
  // RS_LOG(LA_SKILL, "<KickToControlBall> normalize: returning";
  return AngleDeg(a);
}


void KickToControlBall::updWithObjSizes(Float& Sx, Float& Sy)
{
  // RS_LOG(LA_SKILL, "<KickToControlBall> updWithObjSizes: beginning";
  VectorUE t = Vector(Point(Sx, Sy)) + Vector(finalDir, facts->SP_player_size + (1.3) * facts->SP_ball_size);
  // added some costant to avoid just a very little overlapping ------------------|
  
  Sx = ( t.getLength() * cos(t.getAngle().getRad()) ).getValue();
  Sy = ( t.getLength() * sin(t.getAngle().getRad()) ).getValue();

  // RS_LOG(LA_SKILL, "<KickToControlBall> updWithObjSizes: returning";
}


void KickToControlBall::updWithObjSizes(Point& p)
{
  Vector t = Vector(p) + Vector(finalDir, facts->SP_player_size + (1.3) * facts->SP_ball_size);
  // added some costant to avoid just a very little overlapping ------------------|
  
  p.setX( t.getLength() * cos(t.getAngle().getRad()) );
  p.setY( t.getLength() * sin(t.getAngle().getRad()) );
}

Float32 KickToControlBall::requiredDashingPower()
{
  // power usable depands on stamina, effort, recovery, opponents
  // by now i simply use the .30 costant factor.
  Float effort = 1.0;
  Float power = facts->SP_maxpower;
  Float dp_rate = facts->SP_dash_power_rate;
    // RS_LOG(LA_SKILL, "<KickToControlBall> requiredMoment : just returning";
  //return Float32(.75 * power * (1 / (facts->SP_dash_power_rate * effort)));
  return .30 * power * dp_rate;
}

Float KickToControlBall::requiredMoment(AngleDeg& a)
{
  // RS_LOG(LA_SKILL, "<KickToControlBall> requiredMoment : just returning";

 return (agentAbsDir.angleBetween(VectorUE(a, FloatUE(1.0))) *
        (1.0 + facts->SP_inertia_moment * agentView->getSpeed(NEXT))).getValue();
}

RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "KickToControlBall.icc"
#endif
// To here if you do not have any inline definitions!

