/* -*- Mode: C++ -*-
 * $Id: KickToControlBall.h,v 1.3 2002/10/14 15:15:26 aip8 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: KickToControlBall.h
 * Unitname: Framework/skill
 * $Revision: 1.3 $
 * Created by: aip8
 * Last modified by $Author: aip8 $ $Date: 2002/10/14 15:15:26 $
 *
 *
 * HISTORY
 * $Log: KickToControlBall.h,v $
 * Revision 1.3  2002/10/14 15:15:26  aip8
 * *** empty log message ***
 *
 * Revision 1.2  2002/09/29 22:27:30  aip8
 * MoveWithBallToPosition2.cc/.h are more primitive but faster versions of MoveWithBallToPosition.cc/.h
 * The other files are implementations of skills.
 *
 *
 * Revision 0.4  2002/09/27 19:00:00  aip8
 * It should work at least in the simplest cases.
 * Added the normalize private method.
 *
 * Revision 0.1  2002/09/27 00:00:00  aip8
 * It does something not well specified...
 *
 * Revision 0.0  2000/09/18 17:55:02  aip8
 * It compiles, that's all it does.
 *
 * #include rs_log.h
 *
 */

/**
 * @file KickToControlBall.h
 * Include file for the skill class KickToControlBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: aip8 $
 * $Date: 2002/10/14 15:15:26 $
 */


/**
 * @class KickToControlBall KickToControlBall.h
 * This class encapsulates a KickToControlBall skill.
 *
 * <!-- XXXXXXX
 * @pre The preconditions are that we can excpect visual information in
 * The next cycle.
 * @post We find the ball. Or keep on turning
 *
 * @todo This needs to be done...
 * XXXXXXXXXXXXXx
 * --> 
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: aip8 $
 * $Date: 2002/10/14 15:15:26 $
 */


#ifndef KICK_TO_CONTROL_BALL_H
#define KICK_TO_CONTROL_BALL_H


#include "Skill.h"
#include "MathVectorUE.h"

RS_BEGIN_NAMESPACE

class BallView;
class AgentView;
//class GameView;


class KickToControlBall : public Skill
{

public:
  /** Default modulus speed value -1.0 means the current modulus speed.
      Default angle value 360.0 means the current body direction.
  */
  KickToControlBall(Float a = 360.0, Float speed = -1.0); 
  ~KickToControlBall();

  AngleDeg getFinalDirection();

  Plan generatePlan();

  FuzzyBool persistent();

  /**
     Return @c true (close to 1) iff the ball is kickable and it is not
     in the sector of the kickable circle such that its 
   */
  FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  FuzzyBool failed();

  /** KickToControlBall::generatePlan() will compute a plan for
      controlling the ball according to the given argument.
      Default value is FAST, i.e. in the next time step the ball will have
      only the position as better as possible (unspecified speed and accelation). */
  void KickToControlBall::setFinalDir(AngleDeg);
  AngleDeg KickToControlBall::getFinalDir();
  void KickToControlBall::setFinalSpeed(Float);
  Float KickToControlBall::getFinalSpeed();
protected:
  //  GameView * gameView;
  BallView * ballView;
  AgentView * agentView;
  const WorldFacts * facts;
  //isBallKickable should be a predicate!

private:
  /** Return @c the current reduction coefficient describing
      the effective kick power */
  FloatUE ekpCoefficient();

  /** Return the value of alpha according to RoboCup coordinates */
  AngleDeg normalize(AngleDeg);

  void updWithObjSizes(Float& Sx, Float& Sy);
  void updWithObjSizes(Point& p);
  Float32 requiredDashingPower();
  Float requiredMoment(AngleDeg&);

  AngleDeg finalDir;
  Float finalSpeed;

  MathVectorUE agentAbsDir;
};


RS_END_NAMESPACE


#endif // !KICK_TO_CONTROL_BALL_H
