/* -*- Mode: C++ -*-
 * $Id: MakePassable.cc,v 1.7 2002/10/21 18:16:09 aip10 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MakePassable.cc
 * Unitname: Framework/skill
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 2000-09-07
 * Last modified by $Author: aip10 $ $Date: 2002/10/21 18:16:09 $
 *
 *
 * HISTORY
 *
 * $Log: MakePassable.cc,v $
 * Revision 1.7  2002/10/21 18:16:09  aip10
 * Implemented the isOffField method (used by generatePlan)
 *
 * Revision 1.6  2002/10/19 20:37:00  micol
 * Fixed some errors in generatePlan and applicable. Implemented binary search instead of
 * randomization when looking for pass vectors. Added some more checks (ballIsInOurPossesion(),
 * isOffField(), willBeOffside()) (the latter two are just stubs atm). Changed parameter
 * types/names for some functions. Added some comments.
 *
 * Revision 1.5  2002/10/14 09:17:37  aip10
 * seminarie
 *
 * Revision 1.4  2002/10/04 11:29:40  aip10
 * Added first version of skill
 *
 * Revision 1.5  2002/09/13 14:48:56  s02davro
 * FALSE -> FuzzyBool(0.0), TRUE -> FuzzyBool(1.0)
 *
 * Revision 1.4  2001/09/06 13:13:39  andbo928
 * Added class specifier to a lot of methods. Added succeed and failed methods to cc file.
 *
 * Revision 1.3  2000/09/16 18:39:39  frehe
 * Fixed the log messages.
 *
 * Revision 1.2  2000/09/16 18:36:48  frehe
 * Made minor fixes the skill stubs.
 *
 * Revision 1.1  2000/09/07 16:10:34  frehe
 * Added stubs for the skills.
 *
 */

/**
 * @file MakePassable.cc
 * Implementation file for the skill class MakePassable.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: aip10 $
 * $Date: 2002/10/21 18:16:09 $
 */


#include <iostream.h>
#include "MakePassable.h"
#include "AgentView.h"
#include "PlayerView.h"
#include "BallView.h"
#include "GameView.h"
#include "MoveToPosition.h"
#include "view_constants.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

MakePassable::MakePassable()
  // This is an initializer list
  // which means that all the objects inside the
  // class can be initialized after the colon.
  // You initialize the variable myVar with
  // myVar(constructor arguments).
  // You should also initialize the base classes, but
  // only the classes that you directly inherit from.
  // If you do not initialize a variable its default
  // constructor will be called.
  : Skill(),
    agentView(NULL),
    playerView(NULL),
    ballView(NULL),
    gameView(NULL),
    wmi(WorldModelInterface::instance())
{
  agentView = static_cast<AgentView*>(viewManager->getView(AGENT_VIEW_ID));
  if ( agentView == NULL ) {
    agentView = new AgentView(AGENT_VIEW_ID);
    viewManager->addView(agentView, true);
  }
  
  playerView = static_cast<PlayerView*>(viewManager->getView(PLAYER_VIEW_ID));
  if ( playerView == NULL ) {
    playerView = new PlayerView(PLAYER_VIEW_ID);
    viewManager->addView(playerView, true);
  }

  ballView = static_cast<BallView*>(viewManager->getView(BALL_VIEW_ID));
  if ( ballView == NULL ) {
    ballView = new BallView(BALL_VIEW_ID);
    viewManager->addView(ballView, true);
  }
  
  gameView = static_cast<GameView*>(viewManager->getView(GAME_VIEW_ID));
  if ( gameView == NULL ) {
    gameView = new GameView(GAME_VIEW_ID);
    viewManager->addView(gameView, true);
  }
}


// --- DESTRUCTOR ---

MakePassable::~MakePassable()
{
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
}


// --- METHODS ---

Plan MakePassable::generatePlan()
{
  // We need - ballPosition, opp1, opp2, ballPlayerVector
 
  Vector candVector;  // candidate vector
  bool opp1CanIntercept, opp2CanIntercept;
  bool foundGoodPassVector;
  AngleDeg minAngle, maxAngle, rangeAngle;
  Vector vectorXchg;
  PlayerObject playerXchg;
  Float dist1,dist2;

  //cout << "SKILL IS APPLICABLE\n";
  // Order the two opponents so that opponent1 always is the one with the
  // lowest angle (from ballPosition).
  Vector v1(ballPosition.getPoint(), opponent1.getPosition().getPoint());
  Vector v2(ballPosition.getPoint(), opponent2.getPosition().getPoint());
  if (v1.getAngle() > v2.getAngle())
    {
      vectorXchg = v1;
      v1 = v2;
      v2 = vectorXchg;
      playerXchg = opponent1;
      opponent1 = opponent2;
      opponent2 = playerXchg;
    }
  minAngle = v1.getAngle();
  maxAngle = v2.getAngle();
  rangeAngle = v2.getAngle() - v1.getAngle();  
  
  /*
  // Try 10 random candidate vectors in the given range
  // *NOTE* - We might use binary search.
  for (int attempts = 0; attempts < 10; attempts++) {
    //candVector.setAngle(minAngle + (((float)(rand() % 1000) / 1000) * rangeAngle.getDeg()));
    // If neither opp1 nor opp2 can intercept we have found a good vector
    if (!(canIntercept(opp1,candVector,ballPosition.getPoint(),vec1)) &&
	!(canIntercept(opp2,candVector,ballPosition.getPoint(),vec2)))
      break;
  }
  */


  //cout << "minangle: " << minAngle << endl;
  //cout << "maxangle: " << maxAngle << endl;

  // Do a binary search in the given angle range
  foundGoodPassVector = false;
  while (true) 
    {
      // Check if the search is done
      if (maxAngle <= minAngle) break;

      // Place the candidate vector right in the middle
      candVector.setAngle((minAngle+maxAngle)/2);

      dist1 = getOrthDist(opponent1, candVector, ballPosition.getPoint());
      dist2 = getOrthDist(opponent2, candVector, ballPosition.getPoint());

      opp1CanIntercept = canIntercept(&opponent1, candVector, ballPosition.getPoint(), dist1);
      opp2CanIntercept = canIntercept(&opponent2, candVector, ballPosition.getPoint(), dist2);

      // If both opponents can intercept the pass then we know that the pass "corridor" is simply too narrow
      if (opp1CanIntercept && opp2CanIntercept) break;  

      if (opp1CanIntercept)
	{
	  minAngle = candVector.getAngle();  // Move the lower bound "up"
	  continue;
	}else
	if (opp2CanIntercept)
	  {
	    maxAngle = candVector.getAngle(); // Move the upper bound "down"
	    continue;
	  }
      // If we got here, it must be because none of the opponents could intercept the pass
      foundGoodPassVector = true;
      break;
    }
	
  // return an empty plan if no good pass vector was found
  if (!foundGoodPassVector) return Plan();

  // Set the length of the final candidate vector to the length
  // of the ballPlayerVector.
  // *NOTE* - We might use a more optimal length.
  candVector.setLength(ballPlayerVector.getLength());

  Point newPoint = (ballPosition + candVector).getPoint();

  // Do nothing if the new position is off-field
  if (isOffField(newPoint)) return Plan();
  // ..or if we will be offside in our new position
  if (willBeOffside(newPoint)) return Plan();

  // Move to the very end of the final candidate vector.
  MoveToPosition moveToPos((ballPosition + candVector).getPoint());
  return moveToPos.generatePlan();
}


FuzzyBool MakePassable::persistent()
{
  // Never persistent.
  return FuzzyBool(0.0);
}


FuzzyBool MakePassable::applicable()
{
  std::vector<PlayerObject> teamMates = playerView->getTeammates();
  VectorUE ballSpeedVector;
  
  // Precondition - The play mode must be PLAY_ON.
  if (!(gameView->getPlayMode() == PM_PLAY_ON))
    return FuzzyBool(0.0);

  // Precondition - We must know the current position of the ball and it's speed.
  if ((ballView->getPosition().isKnown()) && (ballView->getSpeedVector().isKnown()))
    {
      ballPosition = ballView->getPosition();
      ballSpeedVector = ballView->getSpeedVector();
    } else
      return FuzzyBool(0.0);

  // Precondition - We must know some team mates.
  if (teamMates.empty()) return FuzzyBool(0.0);

  // Precondition - We must not have the ball already
  if (ballIsInOurPossesion()) { cout << "BALL IS ALREADY IN OUR POSSESION\n";  return FuzzyBool(0.0); }

  // #### Variables used for calculating ballPosition ####
  typedef std::vector<PlayerObject> Container;
  Container::iterator i;
  PointUE playerPos;
  Vector playerToBallVector;
  Float requiredCycles = 999;
  const WorldFacts* worldFacts = WorldFacts::instance();
  Float ballDecay = worldFacts->SP_ball_decay;
  Float playerSpeedMax = worldFacts->SP_player_speed_max;
  // #####################################################
  
  int numCycles;
  // First iterate the number of cycles.
  for (numCycles = 0; numCycles < 20; numCycles++)
    {
      i = teamMates.begin();
      
      // Then the players.
      while (i != teamMates.end()) {
	playerPos = (*i).getPosition();
	
	if (playerPos.isKnown()) {
	  playerToBallVector = Vector(playerPos.getPoint(), ballPosition.getPoint());
	  requiredCycles = playerToBallVector.getLength() / playerSpeedMax;
	  
	  if (numCycles >= requiredCycles) break;
	}
	i++;
      }
      
      if (numCycles >= requiredCycles) break;
      ballPosition += ballDecay * ballSpeedVector;
    }

  if (requiredCycles > numCycles)
    // Precondition - We must find a team mate who will catch the ball.
    return FuzzyBool(0.0);

  
  // #### We now have determined from where our teammate will(/might) pass. ####
  
  // Create the ball to player vector
  /*
    B
      \ 
       \        B=Ball
        \       P=Player (the agent)
	 \
	  v
	   P
  */
  Vector BPV(ballPosition.getPoint(), agentView->getPosition().getPoint());
  ballPlayerVector = BPV;  // This is kinda ugly.. ..but it works.

  // Find two opponents, opp1 and opp2, who are othogonally closer
  // to the ballPlayerVector than any other two opponents.
  std::vector<PlayerObject> opponents = playerView->getOpponents();
  PlayerObject opp1, opp2;
  Float d, dist1, dist2;

  // Orthogonal distances from the opponents to the ballPlayerVector.
  // Initially set them to some unreasonably large values
  dist1 = 1000000;
  dist2 = 1000000;
  
  // We must have some opponents
  if (!opponents.empty()) {
    i = opponents.begin();
    opp1 = *i;
    opp2 = *i;

    // Iterates through the list (vector) of known opponents
    while (i != opponents.end()) {
      
      if ((*i).getPosition().isKnown()) {
	d = getOrthDist(*i, ballPlayerVector, ballPosition.getPoint());
	
	if (d < dist1) {
	  opp1 = *i; 
	  opp2 = opp1;
	  dist2 = dist1;
	  dist1 = d;
	}
	else
	  if (d < dist2)	{
	    opp2 = *i;
	    dist2 = d;
	  }
	i++;
      }
    }
  }
  
  if ((dist1 == 1000000) 
      || (dist2 == 1000000)
      // We dont want to choose opponents who are far away.
      || (Vector(opp1.getPosition().getPoint(), agentView->getPosition().getPoint()).getLength()
	  > ballPlayerVector.getLength())
      || (Vector(opp2.getPosition().getPoint(), agentView->getPosition().getPoint()).getLength()
	  > ballPlayerVector.getLength())) {

    // Failed to find two opponents. Do nothing for now...
    return FuzzyBool(0.0); 
  }

  opponent1 = opp1;
  opponent2 = opp2;
  return FuzzyBool(1.0);
}


FuzzyBool MakePassable::succeed()
{
  // Always succedes.
  return FuzzyBool(0.0);
}


FuzzyBool MakePassable::failed()
{
  // Never fails.
  return FuzzyBool(0.0);
}



// Calculates the orthogonal distance from oppPoint to the ballPlayerVector
// PlayerObject* x Vector x Point -> Float
Float MakePassable::getOrthDist(PlayerObject opp, Vector ballPlayerVector, Point ballPoint) 
{
  Vector hypothenuse(opp.getPosition().getPoint(),ballPoint);
  return sin(abs(hypothenuse.getAngle().getRad() - ballPlayerVector.getAngle().getRad())) * hypothenuse.getLength();
}



// Check if an opponent can intercept a pass from ballPosition along the passVector
// 2002-10-18: Changed back to PlayerObject* instead of PlayerObject [micol]
bool MakePassable::canIntercept(PlayerObject *opp, Vector passVector, Point ballPoint, Float oppRunDist)
{
  const WorldFacts* worldFacts = WorldFacts::instance();
  Float ballDecay = worldFacts->SP_ball_decay;

  /*
           ^ 
          /|
    hyp  / | B (|B| = oppRunDist)
        /  |
       /   |
       ------> passVector

  */

  Vector hyp(opp->getPosition().getPoint(),ballPoint);

  // Make a copy of passVector and set its length (it only has an angle atm)
  Vector ourVec(passVector);
  ourVec.setLength(cos(abs(hyp.getAngle().getRad() - passVector.getAngle().getRad())) * hyp.getLength());

  Point intersectionPoint(ballPoint, ourVec);

  // Approximate the time it'll take for the opponent to reach the passVector
  Float timeForOppToRun = oppRunDist / worldFacts->SP_player_speed_max;
  PointUE ballFinalPoint = ballPoint + (timeForOppToRun * ballView->getSpeedVector() * ballDecay);

  // Is the distance to the ball's "final" position greater than to the point where the opponent reaches the
  // passVector ?
  if (Vector(ballFinalPoint.getPoint(),ballPoint).getLength() > Vector(intersectionPoint,ballPoint).getLength())
    // Yep. That means the opponent won't be able to intercept the pass
    return false;
  return true;
}


// Returns true if the new position is off the field
bool MakePassable::isOffField(Point p)
{
  const WorldFacts* worldFacts = WorldFacts::instance(); 

  // Check that the new position isn't outside +/-SP_pitch_length/2,+/-SP_pitch_width/2
  return ((p.getX() < -worldFacts->SP_pitch_length/2) ||
	  (p.getX() > worldFacts->SP_pitch_length/2) ||
	  (p.getY() < -worldFacts->SP_pitch_width/2) ||
	  (p.getY() > worldFacts->SP_pitch_width/2));
}


// Returns true if the agent will become offside while running towards the calculated position (ie. will he
// be offside when his teammate passes him. The time of the pass could be estimated to the time when the teammate
// reaches the ball, plus a few cycles).
bool MakePassable::willBeOffside(Point p)
{
  return false;
}


// Returns true if the ball already is in the agent's possesion.
// (simple check between the agent's position and the ball-position (is the ball within the kickable area))
bool MakePassable::ballIsInOurPossesion()
{
  const WorldFacts* worldFacts = WorldFacts::instance(); 
  // We've already made sure in applicable that the ball-position is known
  Vector v(agentView->getPosition().getPoint(), ballPosition.getPoint());
  
  if (v.getLength() < worldFacts->SP_kickable_area)
    return true;

  return false;
}




RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "MakePassable.icc"
#endif
// To here if you do not have any inline definitions!
