/* -*- Mode: C++ -*-
 * $Id: MakePassable.h,v 1.7 2002/10/21 18:15:12 aip10 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MakePassable.h
 * Unitname: Framework/skill
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 2000-09-07
 * Last modified by $Author: aip10 $ $Date: 2002/10/21 18:15:12 $
 *
 *
 * HISTORY
 *
 * $Log: MakePassable.h,v $
 * Revision 1.7  2002/10/21 18:15:12  aip10
 * Changed some method declarations
 *
 * Revision 1.6  2002/10/19 19:42:51  aip10
 * *** empty log message ***
 *
 * Revision 1.6  2002/10/18 20:00:00  micol
 * Added isOffField and willBeOffside
 *
 * Revision 1.5  2002/10/14 09:17:29  aip10
 * seminarie
 *
 * Revision 1.4  2002/10/04 11:29:40  aip10
 * Added first version of skill
 *
 *
 *
 */

/**
 * @file MakePassable.h
 * Include file for the skill class MakePassable.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: aip10 $
 * $Date: 2002/10/21 18:15:12 $
 */


#ifndef MAKEPASSABLE_H
#define MAKEPASSABLE_H


#include "Skill.h"
#include "WorldModelInterface.h"
#include "WorldFacts.h"
#include "View.h"
#include "WorldModel.h"

RS_BEGIN_NAMESPACE

class AgentView;
class BallView;
class PlayerView;
class GameView;

/**
 * @class MakePassable MakePassable.h
 * The MakePassable skill is intended to be used when we
 * want the player to move to a "passable" position.
 * 
 * @pre There are visiable teammates, the ball is visible, the play mode is PLAY_ON, we don't have the ball ourself already.
 * @post We're on our way to a passable position. 
 * @todo Opponent check...
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: aip10 $
 * $Date: 2002/10/21 18:15:12 $
 */
class MakePassable : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  MakePassable();

  /** Destructor should clean up after the skill. */
  virtual ~MakePassable();

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

  virtual Float getOrthDist(PlayerObject opp, Vector ballPlayerVector, Point ballPoint);
  virtual bool canIntercept(PlayerObject *opp, Vector ballPlayerVector, Point ballPoint, Float oppRunDist);
  virtual bool isOffField(Point);
  virtual bool willBeOffside(Point);
  virtual bool ballIsInOurPossesion();

protected:
  AgentView* agentView;
  PlayerView* playerView;
  BallView* ballView;
  GameView* gameView;

private:
  /** Hide copy-constructor. */
  MakePassable(const MakePassable&);
  /** Hide assignment operator. */
  MakePassable& operator= (const MakePassable&);
  
  const WorldModel* worldModel;
  const WorldModelInterface* const wmi;

  PointUE ballPosition;
  Vector ballPlayerVector;
  PlayerObject opponent1,opponent2;
};


RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "MakePassable.icc"
#endif
// To here if you do not have any inline definitions!

#endif // !SKILLSTUB_H
