/* -*- Mode: C++ -*-
 * $Id: MathVectorUE.cc,v 1.2 2002/10/14 15:15:58 aip8 Exp $Date 2002/09/27 19:00:00 aip8 $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file was created by Rosario Lombardo <roslo679@student.liu.se> and
 * is used in the framework provided by RoboSoc created by
 * Fredrik Heintz <frehe@ida.liu.se>. 
 *
 * IDENTIFICATION
 *
 * Filename: MathVector.cc
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: aip8 $Date
 * Last modified by $Author: aip8 $ $Date 2002/09/27 19:00:00 $
 *
 *
 * HISTORY
 *
 * $Log: MathVectorUE.cc,v $
 * Revision 1.2  2002/10/14 15:15:58  aip8
 * *** empty log message ***
 *
 * Revision 1.1  2002/09/29 22:29:24  aip8
 * MathVectorUE is used by KickToControlBall
 *
 *
 * Revision 1.0  2002/09/27 19:00:00  aip8
 * It provides the methods crossProduct, dotProduct, angleBetween
 * and norm.
 *
 * Revision 0.1  2002/09/24 00:00:00  aip8
 * It does something not well specified...
 *
 */

/**
 * @file MathVector.cc
 * Implementation file for the class MathVector, used in the skill KickToControlBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: aip8 $
 * $Date: 2002/10/14 15:15:58 $
 */


#include "MathVectorUE.h"

RS_BEGIN_NAMESPACE


MathVectorUE::MathVectorUE()
  : VectorUE()
{
}


MathVectorUE::MathVectorUE(VectorUE v)
  : VectorUE(v)
{
}


MathVectorUE::MathVectorUE(AngleDegUE a, FloatUE l)
  : VectorUE(a, l)
{
}

MathVectorUE::~MathVectorUE()
{
}


FloatUE MathVectorUE::crossProduct(VectorUE v)
{
  if (getAngle().isUnknown() || v.getAngle().isUnknown()) // Can i simplify? isUnknown() || v.isUnknown()
    return FloatUE();

  FloatUE x = getLength() * cos(getAngle().getRad());
  FloatUE y = getLength() * sin(getAngle().getRad());
  FloatUE vx = v.getLength() * cos(v.getAngle().getRad());
  FloatUE vy = v.getLength() * sin(v.getAngle().getRad());

  return x * vy - y * vx;
}

  
FloatUE MathVectorUE::dotProduct(VectorUE v)
{
  if (getAngle().isUnknown() || v.getAngle().isUnknown())
    return FloatUE();

  FloatUE x = getLength() * cos(getAngle().getRad());
  FloatUE y = getLength() * sin(getAngle().getRad());
  FloatUE vx = v.getLength() * cos(v.getAngle().getRad());
  FloatUE vy = v.getLength() * sin(v.getAngle().getRad());

  return x * vx + y * vy;
}

/*
AngleRadUE MathVectorUE::angleBetween(VectorUE& v)
{
  FloatUE ang;
  
  ang = crossProduct(v);
  if (dotProduct(v) < 0)
    ang = (ang > 0 ? PI : -PI) - asin(ang.getRad());
  return ang.getValue();
}
*/

FloatUE MathVectorUE::angleBetween(VectorUE v)
{
  FloatUE ang;
  try {
    ang = crossProduct(v);
    if (dotProduct(v) < 0)
      ang = (ang > 0 ? PI : -PI) - asin(ang.getValue());
  } catch (UnknownValueException &uve) {
    return FloatUE();
  }
  //  return ang * PI * (1 / 180);
  return ang;
}


FloatUE MathVectorUE::norm()
{
  if (getAngle().isUnknown() || getLength().isUnknown())
    return FloatUE();

  FloatUE x = getLength() * cos(getAngle().getRad());
  FloatUE y = getLength() * sin(getAngle().getRad());

  return sqrt(x*x + y*y);
}


RS_END_NAMESPACE

