/* -*- Mode: C++ -*-
 * $Id: MoveToPosition.cc,v 1.19 2002/09/26 15:26:52 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveToPosition.cc
 * Unitname: Framework/skill
 * $Revision: 1.19 $
 * Created by: Jan Axelson and Christer Rosenquist 2000-09-24
 * Last modified by $Author: s02davro $ $Date: 2002/09/26 15:26:52 $
 *
 *
 * HISTORY
 * */

/**
 * @file MoveToPosition.cc
 *  
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.19 $
 * $Author: s02davro $
 * $Date: 2002/09/26 15:26:52 $
 */


#include "MoveToPosition.h"
#include "view_constants.h"
#include "Command.h"
#include "TurnCommand.h"
#include "DashCommand.h"
#include "Step.h"
#include "AgentView.h"
#include "GameView.h"


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

MoveToPosition::MoveToPosition(const Point def_position, 
			       const Int def_dash, 
			       const Float def_maxDistance,
			       const AngleDeg def_minTurnAngle)
  : position(def_position),
    dash(def_dash),
    maxDistance(def_maxDistance),
    minTurnAngle(def_minTurnAngle),
    turn(AngleDeg(0)),
    agentView(NULL),
    gameView(NULL) {
  getView(agentView, AGENT_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
}

// --- DESTRUCTOR ---

MoveToPosition::~MoveToPosition() {
  viewManager->releaseView(AGENT_VIEW_ID);
}

// --- METHODS ---

Plan MoveToPosition::generatePlan() {
  static Command* cmd;
  Step step;
  
  AngleDeg turnAngle = (agentView->getPosition().getDirectionTo(getPoint()) -
			agentView->getBodyDirection()).getAngle();
  
  if( abs(turnAngle) > minTurnAngle) {
    cmd = new TurnCommand(turnAngle);
    turn = turnAngle;
  }
  else {
    cmd = new DashCommand(dash); 
    turn = AngleDeg(0);
  }

  step.addCommand(cmd);
  delete cmd;  
  return Plan(step);
}

FuzzyBool MoveToPosition::persistent() {
  return FuzzyBool(0.0);
}

// If the agent's position is known and the agent's distance to the wanna
// go to position is greater than maxDistance, the method returns FuzzyBool(1.0).
FuzzyBool MoveToPosition::applicable() {
  if(agentView->getPosition(NEXT).isKnown() &&
     agentView->getBodyDirection().isKnown() &&
     (agentView->getPosition(NEXT).getDistanceTo(getPoint()) > maxDistance) &&
     gameView->isDashAllowed())
    return FuzzyBool(1.0);
  else
    return FuzzyBool(0.0);
}

void MoveToPosition::setMaxDistance(const Float def_maxDistance) {
  maxDistance = def_maxDistance;
}

Float MoveToPosition::getMaxDistance() const {
  return maxDistance;
}

void MoveToPosition::setMinTurnAngle(const AngleDeg def_minTurnAngle) {
  minTurnAngle = def_minTurnAngle;
}

AngleDeg MoveToPosition::getMinTurnAngle() const {
  return minTurnAngle;
}

void MoveToPosition::setPoint(const Point def_position) {
  position = def_position;
}

Point MoveToPosition::getPoint() const {
  return position;
}

void MoveToPosition::setDash(int def_dash) {
  dash = def_dash;
}

int MoveToPosition::getDash() const {
  return dash;
}

AngleDeg MoveToPosition::getTurn() const {
  return turn;
}

RS_END_NAMESPACE



