/* -*- Mode: C++ -*-
 * $Id: MoveWithBallToPosition.cc,v 1.2 2002/09/29 22:27:31 aip8 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveWithBallToPosition.cc
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: aip8
 * Last modified by $Author: aip8 $ $Date: 2002/09/29 22:27:31 $
 *
 *
 * HISTORY
 * $Log: MoveWithBallToPosition.cc,v $
 * Revision 1.2  2002/09/29 22:27:31  aip8
 * MoveWithBallToPosition2.cc/.h are more primitive but faster versions of MoveWithBallToPosition.cc/.h
 * The other files are implementations of skills.
 *
 * Revision 1.1  2002/09/18 15:14:00  aip8
 * Added initial versions of MoveWithBallToPosition and KickToControlBall skills
 *
 *
 * Revision 0.0  2000/09/18 17:55:02  phigr835
 * It compiles, that's all it does.
 *
 * #include rs_log.h
 *
 */

/**
 * @file MoveWithBallToPosition.cc
 * Implementation file for the skill class MoveWithBallToPosition.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: aip8 $
 * $Date: 2002/09/29 22:27:31 $
 */


#include "MoveWithBallToPosition.h"
#include "AgentView.h"
#include "BallView.h"
#include "GameView.h"
#include "view_constants.h"
#include "TurnCommand.h"
#include "TurnNeckCommand.h"
//Inclusions found in th old skills
#include "rs_log.h"
#include "view_constants.h"
#include "KickCommand.h"
#include "DashCommand.h"

RS_BEGIN_NAMESPACE


MoveWithBallToPosition::MoveWithBallToPosition(const Point& pos)
  : SkillWithPosition(pos),
    ballView(NULL),
    gameView(NULL),
    agentView(NULL),
		persist(false)
    //position(0.0f, 0.0f)
		{  	
  getView(ballView, BALL_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  facts = WorldFacts::instance();
	

}

MoveWithBallToPosition::~MoveWithBallToPosition() {
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
}
//  Methods

//Help method to generate Origin-Vector out of a point
CoordVector<Float> MoveWithBallToPosition::get0Vector(Point p){
  Point hpoint;
  hpoint.setX(0.0);
  hpoint.setY(0.0);
  return hpoint.getVectorTo(p);
  }


/*void MoveWithBallToPosition::setPosition(Point p) {
  position = p;
  }
*/

// speed the player schould have when reaching the ball in percentage[0;1] of the current speed
void MoveWithBallToPosition::setSpeed(Float perc){
	sperc = perc;
	}
  


/* phg
//Ball is kickable or reachable in within  2 cykles
FuzzyBool MoveWithBallToPosition::applicabel(){ // must be adapted
  


  return(ballView->isBallKickable() | );
  }
*///phg
Float MoveWithBallToPosition::q(Float b){
	return b*b;
	}

Plan MoveWithBallToPosition::generatePlan() {

  RS_LOG (LA_SKILL, "MoveWithBallToPostion::generatePlan():");
  Command* cmd;
	// Helping point
	Point hpoint;
	  
  // intended/intermediate Ball's position in the next cykle
  Point iposition;
  Float posx = 0.0;
  Float posy = 0.0;
  //Agent's Postion + Speed
  Float xa;
  xa = agentView->getPosition(NEXT).getPoint().getX(); // ? exact calculation necessary; !depending on speed ! ?
  Float ya;
  ya = agentView->getPosition(NEXT).getPoint().getY(); // ? exact calculation necessary; !depending on speed ! ?
  //Ball's postion
  Float xb;
  xb = ballView->getPosition(NEXT).getPoint().getX(); // ? exact calculation necessary; !depending on speed ! ?
  Float yb;
  yb = ballView->getPosition(NEXT).getPoint().getY(); // ? exact calculation necessary; !depending on speed ! ?
  
  // Ball's direction; towards the intended position d = vbx/vby
  Float d;
  // Agent's planed Speed
  Float vax;
  Float vay;
  Float vax1;
	Float vay1;
	Float vax2;
  Float vay2;

  // Ball's planed speed (Kick power)
  Float vbx;
  Float vby;
  /*Float vbx1; no more used ? 
  Float vby1;
  Float vbx2;
  Float vby2;
	*/
  // kicking angle
  Float kickangle;
  // kicking power
  Float kpow;
  // player's turning angle
  Float turnang;
  //accelleration, needed to make the player go at the wanted speed
  //Float pacc = agentView->getSpeed().getValue()*(sperc - 0.6);   // player's maximal reachable distance; unused 
	Float kickwidth = facts->SP_ball_size+facts->SP_player_size+facts->SP_kickable_margin;
  Float maxreachdist = 3*(agentView->getSpeed().getValue()*sperc);  // ? must be modified with inertia_moment later in this function?
	if(maxreachdist<kickwidth){
	  maxreachdist = kickwidth*1.000001;
		}
	RS_LOG (LA_SKILL, "Speed;sperc  :" <<agentView->getSpeed().getValue()<<";"<<agentView->getSpeedVector().getVector().getLength() << sperc );
  RS_LOG (LA_SKILL, "kickablemargin = ballsize, palyersize, kicklablemargin; maxspeed; maxreachdistance:"<< facts->SP_ball_size<< ";" <<facts->SP_player_size<<";"<<facts->SP_kickable_margin<<";
	"<<facts->SP_player_speed_max <<";"<<maxreachdist );

  // Agent's radius and the intended ballposition: defined either by the reachable positions or the range of movement
  Float r = 0.0;
  if ((get0Vector(agentView->getPosition().getPoint())-get0Vector(position)).getLength() < maxreachdist){
    r = agentView->getPosition().getPoint().getDistanceTo(position);
    posx = position.getX();
    posy = position.getY();
		d = (xb - posx)/(yb - posy);
    }
  else { // position is not reachable within one cykle
     
     // Reckoning of the intended position	
	  Vector dirvect = get0Vector(position) - get0Vector(agentView->getPosition().getPoint()) ;
	  Float l;
 	  Float l1 = (2*(-(pow(xa,2) + posx*xa - pow(ya,2) + posy*ya))  +  2*sqrt( pow( -pow(xa,2)+posx*xa - pow(ya,2)+posy*ya,2
	  ) - (pow(-xa+posx,2)+pow(-ya+posy,2))*(pow(xa,2) + pow(ya,2) - pow(r,2)) ) ) / (2*(pow(-xa+posx,2)+pow(-ya+posy,2))); 
	  Float l2 = (2*(-(pow(xa,2) + posx*xa - pow(ya,2) + posy*ya))  -   2*sqrt( pow( -pow(xa,2)+posx*xa - pow(ya,2)+posy*ya,2
	  ) - (pow(-xa+posx,2)+pow(-ya+posy,2))*(pow(xa,2) + pow(ya,2) - pow(r,2)) ) ) / (2*(pow(-xa+posx,2)+pow(-ya+posy,2))); 
  	/* old:
	   Float l2 = ((ax^2 - posx*ax + ya^2 - posy*ya)  -  2((ax^2-posx*ax + ya^2-posy*ya)^2 - ((ax-posx)^2 +(ya-posy)^2))*(ax^2 + ya^2 - r^2))^0.5) / 2*((ax-posx)^2 +(ya-posy)^2);
	   */
        if ((get0Vector(agentView->getPosition(NEXT).getPoint()) + l1*dirvect - get0Vector(position)).getLength() < (get0Vector(agentView->getPosition(NEXT).getPoint()) + l2*dirvect - get0Vector(position)).getLength()){
      	  l = l1;
				  }
       else {
      	  l = l2;
             }
	  hpoint.setX(cos( (get0Vector(agentView->getPosition(NEXT).getPoint()) + l*dirvect).getAngle().getDeg())*(get0Vector(agentView->getPosition(NEXT).getPoint()) + l*dirvect).getLength() );
		hpoint.setY(sin( (get0Vector(agentView->getPosition(NEXT).getPoint()) + l*dirvect).getAngle().getDeg())*(get0Vector(agentView->getPosition(NEXT).getPoint()) + l*dirvect).getLength() );
	  
	  // old:hpoint.setY(sin((agentView.get0Vector() + l*dirvect)->getAngle())*(agentView.get0Vector() + l*dirvect)->getLength());
        iposition = hpoint;
       //Setting of parameters
       r = maxreachdist;
			 RS_LOG (LA_SKILL, "Values of r :" << r);
       posx = iposition.getX();  // now posx and posy finally are the intended ball's position
       posy = iposition.getY();
       d = (xb - posx)/(yb - posy);
       }
	RS_LOG (LA_SKILL, "Values for equation:" << xa<<ya<<xb<<yb<<"; "<<d<<";"<<r);
	RS_LOG (LA_SKILL, "sqrt:" << (2* pow(d,2)*xa*xb + 2* pow(d,3)*ya*xa - 2*pow(d,3)*yb*xa
         + 2*pow(d,3)*yb*xb - pow(d,4)*pow(ya,2) - pow(d,4)*pow(yb,2) + 2*pow(d,4)*ya*yb - pow(d,2)*pow(xa,2)  - pow(d,2)*pow(xb,2) + pow(d,4)*pow(r,2)
         - 2*pow(d,3)*ya*xb + pow(d,2)*pow(r,2)));
	
	
 
  // Reckoning of the planed Speeds
  vax1 = 1/(2*( pow(d,2)+ 1))*(2*xb - 2*d*yb - 2*xa + 2*d*ya  +  2*sqrt(2* pow(d,2)*xa*xb + 2* pow(d,3)*ya*xa - 2*pow(d,3)*yb*xa
         + 2*pow(d,3)*yb*xb - pow(d,4)*pow(ya,2) - pow(d,4)*pow(yb,2) + 2*pow(d,4)*ya*yb - pow(d,2)*pow(xa,2)  - pow(d,2)*pow(xb,2) + pow(d,4)*pow(r,2)
         - 2*pow(d,3)*ya*xb + pow(d,2)*pow(r,2)));
	vax2 = 1/(2*( pow(d,2)+ 1))*(2*xb - 2*d*yb - 2*xa + 2*d*ya  -  2*sqrt(2* pow(d,2)*xa*xb + 2* pow(d,3)*ya*xa - 2*pow(d,3)*yb*xa
         + 2*pow(d,3)*yb*xb - pow(d,4)*pow(ya,2) - pow(d,4)*pow(yb,2) + 2*pow(d,4)*ya*yb - pow(d,2)*pow(xa,2)  - pow(d,2)*pow(xb,2) + pow(d,4)*pow(r,2)
         - 2*pow(d,3)*ya*xb + pow(d,2)*pow(r,2)));
	RS_LOG (LA_SKILL, "VAX:" << vax1<<";"<<vax2);
			 
  /*oldvax2 = 1/(2*(d^2 + 1))*(2*xb - 2*d*yb - 2*xa + 2*d*ya  -  2*(2*d^2*xa*xb + 2*d^3*ya*xa - 2*d^3*yb*xa
         + 2*d^3*yb*xb - d^4*ya^2 - d^4*yb^2 + 2*d^4*ya*yb - d^2*xa^2  - d^2*xb^2 + d^4*r^2
         - 2*d^3*ya*xb + d^2*r^2)^0.5);
	*/
	// Now q(Float b) is used instead of Pow(Float b,2)
  vay1 = 1/(2*(q(d) + 1)) * (2*d*xa - 2*q(d)*ya + 2*q(d)*yb - 2*d*xb    +    2*sqrt(q(r)  - q(d)*q(ya)  - q(d)*q(yb) - q(xa)
        + 2*xa*xb - q(xb)  + 2*q(d)*ya*yb + 2*d*ya*xa - 2*d*ya*xb - 2*d*yb*xa + 2*d*yb*xb + q(d)*q(r)));
	vay2 = 1/(2*(q(d) + 1)) * (2*d*xa - 2*q(d)*ya + 2*q(d)*yb - 2*d*xb    -    2*sqrt(q(r)  - q(d)*q(ya)  - q(d)*q(yb) - q(xa)
        + 2*xa*xb - q(xb)  + 2*q(d)*ya*yb + 2*d*ya*xa - 2*d*ya*xb - 2*d*yb*xa + 2*d*yb*xb + q(d)*q(r)));
 
  /* old: vay2 = 1/(2*(d^2 + 1)) * (2*d*xa - 2*d^2*ya + 2*d^2*yb - 2*d*xb  -  2*(r^2  - d^2*ya^2  - d^2*yb^2 - xa^2
        + 2*xa*xb - xb^2  + 2*d^2*ya*yb + 2*d*ya*xa - 2*d*ya*xb - 2*d*yb*xa + 2*d*yb*xb + d^2*r^2)^0.5);
  */

  // Selection of the appropiate solutions
	if( q( xa+vax1-posx)+q(ya+vay1-posy) < q( xa+vax2-posx)+q(ya+vay2-posy)){
	  vax = vax1;
	  vay = vay1;
		}
  else {
	  vax = vax2;
  	vay = vay2;
	  }
  vbx = xa-xb+vax;
	vby = ya-yb+vay;
  
    
  //t=0: Calculation of the distance between agent and ball in the cykle(t=2) => determines kickpower
  Float dist;
  Float k;
    //Agent's intended Position, if he would dash to the intended speed
  Float xia = agentView->getPosition(NEXT).getPoint().getX()+agentView->getSpeed().getValue()*sperc*cos(agentView->getSpeedVector().getVector().getAngle());
  Float yia = agentView->getPosition(NEXT).getPoint().getY()+agentView->getSpeed().getValue()*sperc*sin(agentView->getSpeedVector().getVector().getAngle());
    // Player's facing/body direction
  Float facx = cos(agentView->getBodyDirection().getAngle().getDeg());
  Float facy = sin(agentView->getBodyDirection().getAngle().getDeg());
  hpoint.setX(facx);
  hpoint.setY(facy);
  Vector fac = get0Vector(hpoint); 
    //Ball's Position in t=2 cykle, after kicking it = iposition()
  Float bix = iposition.getX();
  Float biy = iposition.getY();
  k = (bix*facx+biy*facy - xia*facx-yia*facy) / (facx * facx + facy * facy);
  dist = (k*fac + get0Vector(agentView->getPosition(NEXT).getPoint())+ agentView->getSpeedVector().getVector()*sperc-get0Vector(iposition)).getLength();

  // selction between kicking, turning, dashing
	if(ballView->isBallKickable()){
	  RS_LOG (LA_SKILL, "Scope : isBallKickable:" << ballView->isBallKickable());
    Float relBallDirection = ballView->getRelativeDirection().getAngle().getDeg();
    Float relBallDistance = ballView->getRelativeDistance().getValue();
		RS_LOG (LA_SKILL, "Scope : isBallKickable:" << ballView->isBallKickable() << "; " << "relBallDirection: " << relBallDirection <<"; "<< "RelBallDistance: "<< relBallDistance  );

      // kickpower depends on, if player has allready got the right direction
    if(dist<facts->SP_ball_size+facts->SP_player_size+facts->SP_kickable_margin){
      kpow = (1+1-facts->SP_ball_decay) * sqrt((pow(vbx,2)+pow(vby,2))) /( facts->SP_kick_power_rate*(1-0.25*relBallDirection/180 - 0.25*(relBallDistance-facts->SP_player_size-facts->SP_ball_size)/facts->SP_kickable_margin) );
			RS_LOG (LA_SKILL, "Scope : kickpow & right direction:" << kpow); 
	    }
    else{
      kpow = pow(1+1 - facts->SP_ball_decay,2) * sqrt((pow(vbx,2)+pow(vby,2))) / (facts->SP_kick_power_rate*(1-0.25*relBallDirection/180 - 0.25*(relBallDistance-facts->SP_player_size-facts->SP_ball_size)/facts->SP_kickable_margin) );
			RS_LOG (LA_SKILL, "Scope : kickpow &  not yet right direction:" << kpow<<vbx<<vby<<"; "<<xa<<xb<<vax<<"Bruch"<<(facts->SP_kick_power_rate*(1-0.25*relBallDirection/180 -
			0.25*(relBallDistance-facts->SP_player_size-facts->SP_ball_size)))<<"; "<<facts->SP_kickable_margin);
      }
    kickangle = get0Vector(iposition).getAngle().getDeg()-get0Vector(ballView->getPosition().getPoint()).getAngle().getDeg();
		RS_LOG (LA_SKILL, "Scope : kickangle:" << kickangle);
    cmd = new KickCommand(kpow, kickangle);
		persist = true;
		} // end first if
	else{
		// t=1 Turning or not Turning
    dist = (k*fac + get0Vector(agentView->getPosition().getPoint())+agentView->getSpeedVector(PREV).getVector()*sperc - get0Vector(iposition)).getLength();
		RS_LOG (LA_SKILL, "Scope : turning?:didtance:" << dist);
    if(dist>facts->SP_ball_size+facts->SP_player_size+facts->SP_kickable_margin){ // Turn
		  turnang = agentView->getPosition().getPoint().getVectorTo(iposition).getAngle().getDeg()-agentView->getBodyDirection().getAngle().getDeg();
			RS_LOG (LA_SKILL, "Scope : turn:turnangle:" << turnang);
	    cmd = new TurnCommand(turnang);
			persist = true;
			}
    else{ //dash
        // Calculation of the accelaration, needed to reach the ball in the next cykle
      //Float acc = ((agentView->getPosition(NEXT).getPoint()).getVectorTo(ballView->getPosition(NEXT).getPoint())).getLength();  
      // Calculation of the dash power-argument => player reaches ball in the next cykle
      Float dashpow = (ballView->getPosition(NEXT).getPoint().getDistanceTo(agentView->getPosition(NEXT).getPoint()))/ (facts->SP_dash_power_rate*agentView->getEffort().getValue());
			RS_LOG (LA_SKILL, "Scope : dash:dashpower:" << dashpow);
			cmd = new DashCommand(dashpow);
			if(iposition != position){
			  persist = true;
				}
    }
  }

  
  Step step;

  //*Do your stuff here
  //cmd = new TurnCommand((agentView->getBodyDirection() - agentView->getNeckDirection()).getDeg());*//
  step.addCommand(cmd);
  delete cmd;
  

  return Plan(step);
}

FuzzyBool MoveWithBallToPosition::persistent() {
  if(persist == true){
	  return FuzzyBool(1.0);
		}
	else{
	  return FuzzyBool(0.0);
		}
 }

FuzzyBool MoveWithBallToPosition::applicable() {
  // chek values on availability
  return (agentView->getPosition(NEXT).isKnown() && ballView->getPosition(NEXT).isKnown() && ballView->isBallKickable());
  }


RS_END_NAMESPACE
