/* -*- Mode: C++ -*-
 * $Id: MoveWithBallToPosition2.cc,v 1.2 2002/10/09 16:11:52 aip8 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveWithBallToPosition2.cc
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: phigr835
 * Last modified by $Author: aip8 $ $Date: 2002/10/09 16:11:52 $
 *
 *
 * HISTORY
 * $Log: MoveWithBallToPosition2.cc,v $
 * Revision 1.2  2002/10/09 16:11:52  aip8
 * Some improvements were made
 *
 * Revision 1.1  2002/09/29 22:27:31  aip8
 * MoveWithBallToPosition2.cc/.h are more primitive but faster versions of MoveWithBallToPosition.cc/.h
 * The other files are implementations of skills.
 *
 * Revision 1.11  2002/09/18 14:40:45  aip8
 * Initial version of MoveWithBallToPosition2
 *
 *
 * Revision 0.0  2002/09/05 21:12:01  Rosario, Marinus, Philip
 * #include rs_log.h
 *
 */

/**
 * @file MoveWithBallToPosition2.cc
 * Implementation file for the skill class MoveWithBallToPosition2.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: aip8 $
 * $Date: 2002/10/09 16:11:52 $
 */


#include "WorldFacts.h"
#include "AgentView.h"
#include "BallView.h"
#include "GameView.h"
#include "PlayerView.h"
#include "view_constants.h"
#include "MoveWithBallToPosition2.h"
#include "GotoBall.h"
#include "KickCommand.h"
#include "TurnCommand.h"

RS_BEGIN_NAMESPACE


MoveWithBallToPosition2::MoveWithBallToPosition2(const Point& pos)
  : SkillWithPosition(pos),
  	worldFacts(NULL),
    ballView(NULL),
    gameView(NULL),
    agentView(NULL),
    playerView(NULL),
		gotoBallSkill(),
		planFinished(false) {
	worldFacts = WorldFacts::instance();
  getView(ballView, BALL_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
}

MoveWithBallToPosition2::~MoveWithBallToPosition2() {
	viewManager->releaseView(BALL_VIEW_ID);
	viewManager->releaseView(GAME_VIEW_ID);
	viewManager->releaseView(AGENT_VIEW_ID);
	viewManager->releaseView(PLAYER_VIEW_ID);
}

void MoveWithBallToPosition2::setSpeed(Float speed){
	Float mySpeed = speed;
	mySpeed++;
}

Plan MoveWithBallToPosition2::generatePlan() {
	static bool lastGotoBall = false;
  Plan plan;
	
	RS_LOG (LA_SKILL, "||MoveWithBallToPosition2::generatePlan():");
	
	if(lastGotoBall && ballView->isBallKickable(NEXT)){
		RS_LOG (LA_SKILL, "||  MoveWithBallToPosition2: plan finished.");
		planFinished = true;
		lastGotoBall = false;
		Step step;
		Command* cmd = new TurnCommand(0);
		step.addCommand(cmd);
		plan.addStep(step);
		delete cmd;
		return plan;
	}
	
	if(agentView->getPosition().getPoint().getDistanceTo(position) < 1.0f){
		RS_LOG (LA_SKILL, "||  MoveWithBallToPosition2: plan almost finished.");
		lastGotoBall = true;
	}else{
		planFinished = false;
	}
	
	if(ballView->isBallKickable() && !lastGotoBall){
		RS_LOG (LA_SKILL, "||  Kick the ball:");
		
		// kick it towards end position
		if(agentView->getBodyDirection().isKnown() && agentView->getPosition().isKnown()){
			Step step;
			Float bodyDirection = agentView->getBodyDirection().getAngle().getDeg();
			Float endPositionDirection = agentView->getPosition().getPoint().getDirectionTo(position).getDeg();
			Float32 power = agentView->getPosition().getPoint().getDistanceTo(position) < 5 ? 5 : 10;
			Command* cmd = new KickCommand(power, endPositionDirection - bodyDirection);
			step.addCommand(cmd);
			plan.addStep(step);
			delete cmd;
		}else{
			RS_LOG (LA_SKILL, "||  MoveWithBallToPosition2: bodydirection or position unknown.");
			Step step;
			Command* cmd = new TurnCommand(0);
			step.addCommand(cmd);
			plan.addStep(step);
			delete cmd;
		}
	}else{
		RS_LOG (LA_SKILL, "||  goto ball:");
		plan = gotoBallSkill.generatePlan();
	}
	
  return plan;
}

/*void setPosition (const Point &pos){
	//planFinished = false;
	SkillWithPosition::setPosition(pos);
}*/

FuzzyBool MoveWithBallToPosition2::persistent(){
  return FuzzyBool(planFinished ? 0.0f : 1.0f);
}

/** Return @c true (close to 1) if the skill has succeeded,
 *  i.e. its post conditions are satisfied.
 */
FuzzyBool MoveWithBallToPosition2::succeed(){
	return FuzzyBool(planFinished ? 1.0f : 0.5f);
}

/** Return @c true (close to 1) if the skill has failed,
 *  i.e. something went wrong.
 */
FuzzyBool MoveWithBallToPosition2::failed(){
	return FuzzyBool(planFinished ? 0.0f : 0.5f);
}

FuzzyBool MoveWithBallToPosition2::applicable(){
	/*
	bool applicable = ballView->isBallKickable();
	if(ballView->getPosition().isKnown() && agentView->getPosition().isKnown()){
		// applicable &= ballView->getPosition().getPoint().getDistanceTo(agentView->getPosition().getPoint()) < worldFacts->SP_kickable_area / 3;
		applicable &= abs(ballView->getPosition().getPoint().getDirectionTo(agentView->getPosition().getPoint()).getDeg()) < 45;
	}else{
		applicable = false;
	}
  return FuzzyBool(applicable ? 1.0f: 0.0f);
	*/
	return ballView->isBallKickable();
}


RS_END_NAMESPACE
