/* -*- Mode: C++ -*-
 * $Id: ReceivePass13.cc,v 1.6 2002/10/15 12:00:43 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ReceivePass13.cc
 * Unitname: Framework/skill
 * $Revision: 1.6 $
 * Created by: Laurent and Yves 2000-09-25
 * Last modified by $Author: s02davro $ $Date: 2002/10/15 12:00:43 $
 *
 *
 * HISTORY
 *
 * $Log: ReceivePass13.cc,v $
 * Revision 1.6  2002/10/15 12:00:43  s02davro
 * Fixed conflicts in RecivePass13
 *
 * Revision 1.5  2002/10/15 11:52:58  aip13
 * *** empty log message ***
 *
 * Revision 1.4  2002/10/15 11:06:47  aip13
 * *** empty log message ***
 *
 * Revision 1.3  2002/10/04 12:42:44  aip13
 * *** empty log message ***
 *
 * Revision 1.2  2002/09/29 22:07:27  aip13
 * *** empty log message ***
 *
 * Revision 1.1  2002/09/25 08:00:37  aip13
 * Added the ReceivePass13 skill.
 *
 *
 */

/**
 * @file ReceivePass13.cc
 * Implementation file for the skill class ReceivePass13.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.6 $
 * $Author: s02davro $
 * $Date: 2002/10/15 12:00:43 $
 */


#include "ReceivePass13.h"
#include "AgentView.h"
#include "BallView.h"
#include "PlayerView.h"
#include "view_constants.h"
#include "WorldFacts.h"
#include "TurnCommand.h"
#include "DashCommand.h"
#include "PlayerObject.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

ReceivePass13::ReceivePass13()
  : Skill(),
    agentView(NULL),
    ballView(NULL),
    playerView(NULL),
    facts(WorldFacts::instance()){
  getView(agentView, AGENT_VIEW_ID);
  getView(ballView, BALL_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
}


// --- DESTRUCTOR ---

ReceivePass13::~ReceivePass13() {
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
}


// --- METHODS ---


Plan ReceivePass13::agentIsWaiting(){
  static Command* cmd;
  Step step;

  cmd = new TurnCommand(0);
  step.addCommand(cmd);
  delete cmd;
  return Plan(step);
}

Point ReceivePass13::agentToBallInter()
{
  Point result = Point(100,100);

  int tBall = 0;
  Float tAgent=1000;
  Float dist_ball=0;
  Float dist_ballToAgent=0;
  while (tAgent>tBall){
    tBall++;
    dist_ball = ballView->getSpeedVector().getVector().getLength()*(1-pow(facts->SP_ball_decay, tBall))/(1-facts->SP_ball_decay);
    dist_ballToAgent = agentView->getPosition().getPoint().getDistanceTo((ballView->getPosition() + (dist_ball * ballView->getSpeedVector())).getPoint());
    tAgent = dist_ballToAgent/facts->SP_player_speed_max;
  }
  result = Point(ballView->getPosition(tBall).getPoint());
  return result;
}


Plan ReceivePass13::generatePlan() {
  Plan result = agentIsWaiting();
  Step step;
  
  //moveToPosAsap = new MoveToPosAsap(Point(0,0), AngleDeg(10.0));
  
  if(ballView->getSpeedVector().getLength() < 0.2 * facts->SP_ball_speed_max){
    //the ball is stopped or nearly stopped
    //std::cout << "        The ball is ~stopped" << std::endl;
    moveToPosAsap->setPoint(ballView->getPosition().getPoint()); 
    //std::cout << "        We launch the MoveToPosAsap skill" << std::endl; 
    result = moveToPosAsap->generatePlan(); 
    
  } else {
    //std::cout << "        The ball is moving" << std::endl;
    //Test to go to the ball asap if an opponent is closer to the ball than the agent
    if(playerView->getOpponentClosestToBall().getPosition().getDistanceTo(ballView->getPosition()) < ballView->getRelativeDistance()){
      moveToPosAsap->setPoint(agentToBallInter());
      result = moveToPosAsap->generatePlan();
    } else {
      //We compute a scalar product to know whether the ball is going towards the opponent's goal or not.
      Vector ball_to_goal_vect = Vector(ballView->getPosition().getPoint(), facts->getTheirGoal().getPosition());
      Float ball_goal_scal_prod = (Point(ballView->getSpeedVector().getVector()).getX() *  Point(ball_to_goal_vect).getX()) +
	(Point(ballView->getSpeedVector().getVector()).getY() *  Point(ball_to_goal_vect).getY());
      
      //Test if the ball is going towards the goal
      if(ball_goal_scal_prod > 0) {

	//We compute where is the ball going to stop naturally
	Float t_ball_stop = log10(facts->SP_ball_speed_max/(10 * ballView->getSpeedVector().getLength().getValue()));
	
	Point ballPositionStop = ballView->getPosition(int(ceil(t_ball_stop))).getPoint();
	
	//We calculate which opponent is closer to the position the ball is going to stop
	std::vector<PlayerObject> opp = playerView->getOpponents();
	Float closestOppToPoint = 1000;
	PointUE closestOppPosition = PointUE(0,0);
	for (std::vector<PlayerObject>::const_iterator addo = opp.begin(); addo != opp.end(); ++addo ) {
	  if(addo->getPosition().isKnown()){
	    if(closestOppToPoint > addo->getPosition().getDistanceTo(ballPositionStop).getValue()){
	      closestOppToPoint = addo->getPosition().getDistanceTo(ballPositionStop).getValue();
	      closestOppPosition = addo->getPosition();
	    }
	  }
	}
	
	//We test if the the ball will be closer to an opponent than the agent at that position (when ball stopped)
	//If point closer to agent
	if(agentView->getPosition().getDistanceTo(PointUE(ballPositionStop)) < closestOppToPoint){
	  moveToPosAsap->setPoint(ballPositionStop);
	  result = moveToPosAsap->generatePlan();
	  
	  //If not we try to find a limit point to go to reach the ball before the opponent
	} else {
	  int tBall = 0;
	  Float tAgent=1000;
	  Float dist_agent_to_ball = agentView->getPosition().getDistanceTo(ballView->getPosition(NOW)).getValue();
	  Float dist_opp_to_ball = closestOppPosition.getDistanceTo(ballView->getPosition(NOW)).getValue();
	  while (tAgent>tBall){
	    if(dist_agent_to_ball == (dist_opp_to_ball + facts->SP_kickable_margin)){
	      break;
	    }
	    tBall++;
	    dist_agent_to_ball = agentView->getPosition().getDistanceTo(ballView->getPosition(tBall)).getValue();
	    dist_opp_to_ball = closestOppPosition.getDistanceTo(ballView->getPosition(tBall)).getValue();
	  }
	  moveToPosAsap->setPoint(ballPositionStop);
	  result = moveToPosAsap->generatePlan();
	}
	
      } else {
	//Perpendicular case
	
	//We calculate the perpendicular intersection of the ball and the player
	FloatUE coeff = (Point(ballView->getSpeedVector(NOW).getVector()).getY() * 
			 (agentView->getPosition(NOW).getY() - ballView->getPosition(NOW).getY()) +
			 Point(ballView->getSpeedVector(NOW).getVector()).getX() *
			 (agentView->getPosition(NOW).getX() - ballView->getPosition(NOW).getX())) /
	  (pow(Point(ballView->getSpeedVector(NOW).getVector()).getX(),2) + 
	   pow(Point(ballView->getSpeedVector(NOW).getVector()).getY(),2));
	
	if (coeff > 0){
	  //std::cout << "        The ball is coming towards the agent" << std::endl;
	  // ball is coming towards the agent
	  
	  //compute the intersection
	  FloatUE inter_x = Point(ballView->getSpeedVector(NOW).getVector()).getX() * coeff +
	    ballView->getPosition(NOW).getX();
	  
	  FloatUE inter_y = Point(ballView->getSpeedVector(NOW).getVector()).getY() * coeff +
	    ballView->getPosition(NOW).getY();
	  
	  //Test if the player can be at the intersection point before the ball.
	  
	  //First we need to calculate the two lenght of times for the ball and the agent to reach the intersection point.
	  // first the agent model: speed = player_speed_max
	  FloatUE agent_dist_to_inter = sqrt(pow((inter_x - agentView->getPosition().getX()).getValue(),2) + pow((inter_y - agentView->getPosition().getY()).getValue(),2));
	  
	  FloatUE agent_time_to_inter = agent_dist_to_inter/facts->SP_player_speed_max;
	  
	  // now the ball model (geometric suite)
	  FloatUE dist_ball = (ballView->getSpeedVector().getLength()) * (1 - pow(facts->SP_ball_decay,agent_time_to_inter.getValue()))/(1 - facts->SP_ball_decay);
	  FloatUE dist_ball_inter = Point(inter_x.getValue(), inter_y.getValue()).getDistanceTo(ballView->getPosition().getPoint());
	  
	  //We test if we can reach the "inter" point before the ball does. 
	  if(dist_ball_inter >= dist_ball){
	    moveToPosAsap->setPoint(Point(inter_x.getValue(), inter_y.getValue()));
	    result = moveToPosAsap->generatePlan();
	    
	    //We cannot reach the ball at the inter prependicular point
	  } else {
	    Point agentToBall = agentToBallInter();
	    if(agentToBall != Point(100,100)){
	      moveToPosAsap->setPoint(agentToBall);
	      result = moveToPosAsap->generatePlan();
	    } else {
	      result = agentIsWaiting();
	    }
	  }
	}
      }
    }
  }
  return result;
}


FuzzyBool ReceivePass13::persistent()
{
  // Never persistent.
  return FuzzyBool(0.0);
}


FuzzyBool ReceivePass13::applicable()
{
  FuzzyBool result = FuzzyBool(0.0);
  
  if (agentView->getPosition().isKnown() && 
      ballView->getPosition().isKnown() && 
      ballView->getSpeedVector().isKnown() && 
      ballView->getRelativeDistance().isKnown() &&
      playerView->getTeammateClosestToBall().getPosition().isKnown() &&
      playerView->getOpponentClosestToBall().getPosition().isKnown() &&
      moveToPosAsap->applicable()){
    //Test if a teammate is closer to the ball than the agent
    if((playerView->getTeammateClosestToBall().getPosition().getDistanceTo(ballView->getPosition()) > ballView->getRelativeDistance())){
      result = FuzzyBool(1.0);
    }
  }
  return result;
}

FuzzyBool ReceivePass13::succeed()
{
  // Always succedes.
  return FuzzyBool(0.0);
}

FuzzyBool ReceivePass13::failed()
{
  // Never fails.
  return FuzzyBool(0.0);
}


RS_END_NAMESPACE
