/* -*- Mode: C++ -*-
 * $Id: Scoreaip6.h,v 1.1 2002/09/27 12:56:17 aip6 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Score.h
 * Unitname: Framework/skill
 * $Revision: 1.1 $
 * Created by: AIP6, 2002-09-11
 * Last modified by $Author: aip6 $ $Date: 2002/09/27 12:56:17 $
 *
 *
 * HISTORY
 *
 * $Log: Scoreaip6.h,v $
 * Revision 1.1  2002/09/27 12:56:17  aip6
 *
 * Initial version.
 *
 */

/**
 * @file Score.h
 * Include file for the skill class Score.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: aip6 $
 * $Date: 2002/09/27 12:56:17 $
 */

#ifndef SCORE_H
#define SCORE_H


#include "Skill.h"

RS_BEGIN_NAMESPACE

class BallView;
class AgentView;
class PlayerView;
class WorldFacts;

/**
 * @class Score Score.h
 * The skill score is used to score a goal.
     
 * @pre The preconditions for the skill are ...
 * @post The intended postconditions for the skill are...
 *
 * @todo Things that should be done...
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: aip6 $
 * $Date: 2002/09/27 12:56:17 $
 */

class Score : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  Score();

  /** Destructor should clean up after the skill. */
  virtual ~Score();

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();
  
private:
  /** Hide copy-constructor. */
  Score(const Score&);
  /** Hide assignment operator. */
  Score& operator= (const Score&);

  const WorldFacts* facts;

  BallView* ballView;
  AgentView* agentView;
  PlayerView* playerView;

  /** This method recalculates and updates the internal aimpoint and
    calculates the probability to score*/
  void updateAimPoint();

  /** The current aimpoint */
  Point aimPoint;

  /** The current probability to score */
  FuzzyBool hitProbability;

  /** TRUE if the aim is updated this cycle */
  bool updatedAim;
};


RS_END_NAMESPACE

#endif // !SCORE_H

