/* -*- Mode: C++ -*-
 * $Id: SkillWithZone.cc,v 1.16 2002/10/12 21:19:46 aip4 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SkillWithPosition.cc
 * Unitname: Framework
 * $Revision: 1.16 $
 * Created by: Fredrik Heintz 2000-09-09
 * Last modified by $Author: aip4 $ $Date: 2002/10/12 21:19:46 $
 *
 *
 * HISTORY
 *
 * $Log: SkillWithZone.cc,v $
 * Revision 1.16  2002/10/12 21:19:46  aip4
 *
 * Modified to a more working state. :)
 *
 * Revision 1.1  2000/09/09 11:53:05  frehe
 * Added four new abstract skill classes, SkillWithDirection, Vector, Position and Speed.
 *
 */

/**
 * @file SkillWithPosition.cc
 * Implementation file for the class SkillWithPosition.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.16 $
 * $Author: aip4 $
 * $Date: 2002/10/12 21:19:46 $
 */


#include "SkillWithZone.h"
#include "rs_throw.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

SkillWithZone::SkillWithZone(const Rectangle& rect)
  : zone(rect)
{
}


// --- DESTRUCTOR ---

SkillWithZone::~SkillWithZone()
{
}


// Simple members

void SkillWithZone::setZone(const Rectangle& rect)
{
  zone = rect;
}

Rectangle SkillWithZone::getZone() const
{
  return zone;
}

void SkillWithZone::setArguments(const std::vector<Float>& arguments) throw(TooFewArgumentsException)
{
  if (arguments.size() < 4)
    {
      RS_THROW(TooFewArgumentsException,
	       "SkillWithZone got less than one argument.");
    }
  else {
    zone = Rectangle(arguments[0], arguments[1], arguments[2], arguments[3]);
  }
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "SkillWithZone.icc"
#endif
