
/**
 * @file SkillWithZone.h
 * Include file for the class SkillWithZone.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.20 $
 * $Author: aip4 $
 * $Date: 2002/10/12 21:19:46 $
 */


#ifndef SKILLWITHZONE_H
#define SKILLWITHZONE_H


#include "SkillWithArguments.h"
#include "TooFewArgumentsException.h"
#include "library_types.h"


RS_BEGIN_NAMESPACE


/**
 * @class SkillWithZone SkillWithZone.h
 * This class represent a skill that has a defined area in which to operate.
 *
 * @par Unit:
 *      Framework
 */
class SkillWithZone : public SkillWithArguments {
public:
  /** Constructor taking the default argument for the Zone (A Rectangle)
    used as an argument in the skill. */
  SkillWithZone(const Rectangle& rect);

  /** Destructor cleaning up after the skill. */
  virtual ~SkillWithZone();


  /** Used to set the skills's arguments in a generic way.
    The values in the vector will be interpreted as:
    arg1 Upper-left x-coordinate 
    arg2 Upper-left y-coordinate
    arg3 Lower-right x-coordinate
    arg4 Lower-right y-coordinate */
    
  virtual void setArguments(const std::vector<Float>& arguments)
     throw(TooFewArgumentsException);


  /** Used to set the skill's zone argument to @a rect. */
  virtual void setZone(const Rectangle& rect);

  /** Return the skill's zone argument. */
  virtual Rectangle getZone() const;

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. Abstract since
    no concrete instances of this class should be created. */
   virtual Plan generatePlan() = 0;

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. Abstract since
    no concrete instances of this class should be created. */
  virtual FuzzyBool persistent() = 0;

  /** This method should implement the precondition specified in
   the description of the skill. Abstract since no concrete instances
   of this class should be created. */
  virtual FuzzyBool applicable() = 0;

protected:  
  /** The zone argument. */
  Rectangle zone;

  
private:
  /** Hide copy-constructor. */
  SkillWithZone(const SkillWithZone&);
  /** Hide assignment operator. */
  SkillWithZone& operator= (const SkillWithZone&);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

/*#ifdef RS_USE_INLINE
  #  include "SkillWithZone.icc"
  #endif */


#endif // !SKILLWITHPOSITION_H
