/* -*- Mode: C++ -*-
 * $Id: Spread.h,v 1.3 2002/10/14 17:22:35 aip1 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Spread.h
 * Unitname: Framework/skill
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-09-07
 * Last modified by $Author: aip1 $ $Date: 2002/10/14 17:22:35 $
 *
 *
 * HISTORY
 *
 * $Log: Spread.h,v $
 * Revision 1.3  2002/10/14 17:22:35  aip1
 * Added doxygen comments.
 *
 * Revision 1.2  2002/10/13 11:50:02  aip1
 * Cleaned up the Spread skill.
 *
 * Revision 1.1  2002/09/20 14:38:57  aip1
 * Added the Spread skill.
 *
 * Revision 1.5  2001/09/06 13:13:39  andbo928
 * Added class specifier to a lot of methods. Added succeed and failed methods to cc file.
 *
 * Revision 1.4  2001/09/01 14:45:12  frehe
 * Updated the skill stub to the new abstract Skill class.
 *
 * Revision 1.3  2000/09/16 18:39:39  frehe
 * Fixed the log messages.
 *
 * Revision 1.2  2000/09/16 18:36:48  frehe
 * Made minor fixes the skill stubs.
 *
 * Revision 1.1  2000/09/07 16:10:34  frehe
 * Added stubs for the skills.
 *
 */

/**
 * @file Spread.h
 * Include file for the skill class Spread.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: aip1 $
 * $Date: 2002/10/14 17:22:35 $
 */


#ifndef SPREAD_H
#define SPREAD_H


#include "Skill.h"

RS_BEGIN_NAMESPACE

class AgentView;
class BallView;
class GameView;
class PlayerView;

/**
 * @class Spread Spread.h
 * A class for a spread skill.
 * This skill makes sure teammates aren't too close to each other.
 *
 * @pre The preconditions are that the player is not the team player 
 * closest to the ball, and that at least N other teammates are 
 * within a radius R of the player.
 * @post The postcondition for the skill is that the player is moving 
 * in a direction away from the other players.
 *
 * @todo
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: aip1 $
 * $Date: 2002/10/14 17:22:35 $
 */
class Spread : public Skill {
public:
  /** Constructor. Takes no argument. */
  Spread();

  virtual ~Spread();
  
  /** Generates a plan for moving away from the other players. */
  virtual Plan generatePlan();

  virtual FuzzyBool persistent();

  /** Checks the preconditions using the given arguments. */
  virtual FuzzyBool applicable(float radius,int allowedPlayers);
  /** Applicable check using default values for radius (5.0) and 
      allowedPlayers (1). */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

  
private:
  Spread(const Spread&);
  Spread& operator= (const Spread&);
  /** Minimum allowed distance between players. Set in applicable(), used 
      again in generatePlan(). */
  float radius;
  
protected:
  BallView* ballView;
  GameView* gameView;
  AgentView* agentView;
  PlayerView* playerView;
};


RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "Spread.icc"
#endif
// To here if you do not have any inline definitions!

#endif // !SPREAD_H
