/* -*- Mode: C++ -*-
 * $Id: AgentView.cc,v 1.9 2002/08/31 15:22:45 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AgentView.cc
 * Unitname: Framework/view
 * $Revision: 1.9 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2002/08/31 15:22:45 $
 *
 *
 * HISTORY
 *
 * $Log: AgentView.cc,v $
 * Revision 1.9  2002/08/31 15:22:45  s02davro
 * Small changes in views
 *
 * Revision 1.8  2002/08/02 21:53:01  s02davro
 * Now works by extracting information from the AgentObject in the WorldModel, parameterView depenency removed.
 *
 * Revision 1.7  2001/10/14 21:21:44  frehe
 * Added better support for messages from self and from other players.
 *
 * Revision 1.6  2001/09/17 21:26:21  andbo928
 * Now uses the SpeedDirection value supplied by the sserver 7 protocol.
 *
 * Revision 1.5  2001/01/11 22:00:20  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.4  2001/01/10 15:12:58  olalu526
 * Added support for WorldModel to some basic classes.
 *
 * Revision 1.3  2000/10/18 23:06:35  frehe
 * Added lastCoachMessage and lastPlayerMessage.
 *
 * Revision 1.2  2000/09/25 13:34:18  frehe
 * Made minor modifications.
 *
 * Revision 1.1  2000/09/21 13:46:45  frehe
 * Move the RoboSoc views from rsviews to the rsframework library.
 *
 * Revision 1.3  2000/09/13 00:08:05  frehe
 * Updated the views to inherit from the ViewWithHistory instead and also 
 * changed those views that used a static pointer to itself to do process 
 * the data in the sensor interface to use the new forEach methods 
 * implemented in the FrameworkSensorInterface.
 *
 * Revision 1.2  2000/09/05 21:15:57  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/04 15:27:18  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file AgentView.cc
 * Implementation file for the class AgentView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.9 $
 * $Author: s02davro $
 * $Date: 2002/08/31 15:22:45 $
 */

#include "AgentView.h"
#include "view_constants.h"
#include "rs_error.h"
#include "rs_log.h"

RS_BEGIN_NAMESPACE

AgentView::AgentView(const ViewId id)
  : View(id) {
}

AgentView::~AgentView() {
}

bool AgentView::isAgentOutsideField() const {
  static const Float offset_x = 0.75;
  static const Float offset_y = 0.75;
  const WorldFacts* facts = worldModelInterface->getWorldFacts();
  
  PointUE apos = getPosition(NOW);
  if ( (apos.getX().isKnown() &&
	abs(apos.getX()) > (facts->SP_pitch_length/2 + offset_x )) ||
       (apos.getY().isKnown() &&
	abs(apos.getY()) > (facts->SP_pitch_width/2 + offset_y )) ) {
    return true;
  }
  return false;
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "AgentView.icc"
#endif
