/* -*- Mode: C++ -*-
 * $Id: FrameworkSensorInterface.h,v 1.5 2002/08/31 15:27:02 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FrameworkSensorInterface.h
 * Unitname: BasicSystem
 * $Revision: 1.5 $
 * Created by: David Rosen 2002-06-18
 * Last modified by $Author: s02davro $ $Date: 2002/08/31 15:27:02 $
 *
 *
 * HISTORY
 *
 * $Log: FrameworkSensorInterface.h,v $
 * Revision 1.5  2002/08/31 15:27:02  s02davro
 * New modified startup, agent, frameworksensorinterface updated
 *
 * Revision 1.4  2002/08/02 22:40:49  s02davro
 * Uses the new Sensor version, WorldObservation and WorldStateInterface.
 *

 */

/**
 * @file FrameworkSensorInterface.h
 * Include file for the class FrameworkSensorInterface.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2002/08/31 15:27:02 $
 */


#ifndef FRAMEWORKSENSORINTERFACE_H
#define FRAMEWORKSENSORINTERFACE_H


#include "basic_system_types.h"
#include "SensorInterface.h"
#include "WorldModelInterface.h"
#include "WorldObservation.h"
#include "WorldFacts.h"
#include "Time.h"
#include <vector>

RS_BEGIN_NAMESPACE


/**
 * @class FrameworkSensorInterface FrameworkSensorInterface.h
 * This class is a implementation of the abstract SensorInterface class.
 * The information from the sensors is given error intervalls using
 * the known information about server noise and then passed to the
 * worldModelInterface in a WorldObservation object to be included
 * in the agents WorldModel.
 * 
 * @todo Check error values.
 * @todo Maybe the FrameworkSensorInterface should be responsible
 *       for calling the update functions in the ViewManager.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2002/08/31 15:27:02 $
 */

class FrameworkSensorInterface : public SensorInterface {
public:
  /** Default constructor. */
  FrameworkSensorInterface();
  
  /** Desturctor for the class. */
  virtual ~FrameworkSensorInterface();

  /** Is called when we have a new cycle. */
  void newCycle();

  /** Called before a new cycle is issued. */
  void updateBeforeNewCycle();

  /** Sets the last command executed by the agent. */
  void setLastCommand(ServerCommand *command);

  /** Called from Sensors when we have new visual information. */
  void see(const std::vector<ServerLineObject>& lines,
	   const std::vector<ServerMarkerObject>& markers,
	   const std::vector<ServerPlayerObject>& players,
	   const ServerBallObject& ball);

  /** Called from Sensors when we have new global visual information.(coach) */
  void seeGlobal(const std::vector<CoachPlayerObject>& cplayers,
		 const CoachBallObject& cball);

  /** Called from Sensors when we have new sense information. */
  void sense(const ServerAgentObject& agent);

  /** Called when we hear the referee. */
  void hearReferee(const PlayMode play_mode);

  /** Called when we hear a message from our coach. */
  void hearCoach(const std::string& message);

  /** Called when we hear a message from ourself. */
  void hearSelf(const std::string& message);

  /** Called when we hear another player. */
  void hearPlayer(const AngleDegUE msg_dir, const std::string& message);

  /** Called when we hear another (known by nr.) player. (coach) */
  virtual void hearPlayer(const UInt unum,
			  const std::string& message);

  /** Sets the last command executed by the agent. */
  void setLastSensorType(const SensorType sensor_type);
  
protected:  
  /** A pointer to the agents worldModelInterface.
   *  This is used to pass WorldObservation objects to reflect the new
   *  sensor information about the world. */
  WorldModelInterface *worldModelInterface;

  /** The WorldObservation for the current cycle. */
  WorldObservation obsWorld;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "FrameworkSensorInterface.icc"
#endif


#endif // !FRAMEWORKSENSORINTERFACE_H
