/* -*- Mode: C++ -*-
 * $Id: WorldObservation.h,v 1.2 2002/08/31 15:31:18 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: WorldObservation.h
 * Unitname: BasicSystem
 * $Revision: 1.2 $
 * Created by: David Rosen 2002-06-18
 * Last modified by $Author: s02davro $ $Date: 2002/08/31 15:31:18 $
 *
 *
 * HISTORY
 *
 * $Log: WorldObservation.h,v $
 * Revision 1.2  2002/08/31 15:31:18  s02davro
 * Updated the world model
 *
 * Revision 1.1  2002/08/02 21:59:20  s02davro
 * Stores the information in a new observation made by the agent.
 *
 */

/**
 * @file WorldObservation.h
 * Include file for the class WorldObservation.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2002/08/31 15:31:18 $
 */


#ifndef WORLDOBSERVATION_H
#define WORLDOBSERVATION_H


#include "basic_system_types.h"
#include "ServerCommand.h"
#include "Time.h"

RS_BEGIN_NAMESPACE


/**
 * @class WorldObservation WorldObservation.h
 * Stores the information from a new observation made by the agent.
 * This is initialized in the FrameworkSensorInterface and
 * sent to the WorldModelInterface, used by the Strategies to
 * create a new better WorldModel for the agent.
 * 
 * @todo More information may be needed here.
 * @todo Copyconstructor and assignment operator.
 *
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2002/08/31 15:31:18 $
 */

class WorldObservation {
public:
  /** Constructor */
  WorldObservation();  

  /** Destructor. */
  virtual ~WorldObservation();

  /** Clear all information. */
  void clear();

  /** The observed playmodes. */
  std::vector<PlayMode> playModes;

  /** The observed information about the ball. */
  ServerBallObject ball;

  /** Observed (sensed) information aobut the agent itself. */
  ServerAgentObject agent;

  /** Observed information about other players. */
  std::vector<ServerPlayerObject> players;  

  /** Observed markers. */
  std::vector<ServerMarkerObject> markers;

  /** Observed lines. */
  std::vector<ServerLineObject> lines;

  /** Message heard from ourself. */
  std::string ownMessage;

  /** Message heard from another player. */
  std::string playerMessage;

  /** Message heard from the coach. */
  std::string coachMessage;
  
  /** Direction of the heard playermessage */
  AngleDegUE messageDirection;

  /** The observed time for the game. */
  Time gameTime;
  
  /** A coachs observation of the ball. */
  CoachBallObject cball;

  /** A coachs observation about the players. */
  std::vector<CoachPlayerObject> cplayers;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "WorldObservation.icc"
#endif


#endif // !WORLDOBSERVATION_H
