/* -*- Mode: C++ -*-
 * $Id: BasicNewCycleStrategy.cc,v 1.5 2002/09/04 07:55:40 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BasicNewCycleStrategy.cc
 * Unitname: BasicSystem
 * $Revision: 1.5 $
 * Created by: David Rosn 2002-07-01
 * Last modified by $Author: s02davro $ $Date: 2002/09/04 07:55:40 $
 *
 *
 * HISTORY
 *
 * $Log: BasicNewCycleStrategy.cc,v $
 * Revision 1.5  2002/09/04 07:55:40  s02davro
 * Speedfix
 *
 * Revision 1.4  2002/09/03 18:04:29  s02davro
 * Speed and NewCycle fix
 *
 * Revision 1.3  2002/09/03 10:57:39  s02davro
 * LastSeeTime, NextSeeTime, NextNextSeeTime
 *
 * Revision 1.2  2002/08/31 15:35:24  s02davro
 * Updated the strategies and the interface
 *
 * Revision 1.1  2002/08/02 22:00:52  s02davro
 * Created the Basic classes, needs improvements.
 *
 */

/**
 * @file BasicNewCycleStrategy.cc
 * Implementation file for the class BasicNewCycleStrategy.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2002/09/04 07:55:40 $
 */

#include "BasicNewCycleStrategy.h"
#include "WorldModelInterface.h"
#include "rs_error.h"

RS_BEGIN_NAMESPACE

BasicNewCycleStrategy::BasicNewCycleStrategy() {
}

BasicNewCycleStrategy::~BasicNewCycleStrategy() { 
}

void BasicNewCycleStrategy::updateBeforeNewCycle(WorldModel &def_world) {  
  RS_LOG ( LA_STRATEGY, "BasicNewCycleStrategy - begin");  
  world = &def_world;
  AgentObject agent = world->getAgent();
  BallObject ball = world->getBall();
  VectorUE moveSpeed;
  world->setGameTime(world->getGameTime() + 1);
  
  // Agent
  if (agent.getTurn().isKnown()) {
    agent.setBodyDirection(agent.getBodyDirection() + agent.getTurn());  
    agent.setNeckDirection(agent.getNeckDirection() + agent.getTurn());  
  }
  if (agent.getTurnNeck().isKnown()) {
    agent.setNeckDirection(agent.getNeckDirection() + agent.getTurnNeck());
  }
  if (agent.getSpeedVector().isKnown()) {
    moveSpeed = agent.getSpeedVector();
    if (agent.getAcceleration().isKnown()) {
      moveSpeed += agent.getAcceleration();
    }
    agent.setPosition(agent.getPosition() + moveSpeed);
    agent.setSpeedVector(moveSpeed * facts->SP_player_decay);
  }

  // If the agent made a move command.
  if (agent.getMovePosition().isKnown()) {
    agent.setPosition(agent.getMovePosition());
    agent.setSpeedVector(VectorUE(0,0));
  }  
  
  // Ball
  if (ball.getSpeedVector().isKnown()) {
    moveSpeed = ball.getSpeedVector();
    if (ball.getAcceleration().isKnown()) {
      moveSpeed += ball.getAcceleration(); 
    }   
    ball.setPosition(ball.getPosition() + moveSpeed);
    ball.setSpeedVector(moveSpeed * facts->SP_ball_decay);
  }
      
  // Players
  for (std::vector<PlayerObject>::iterator i = world->getTeammates().begin();
       i != world->getTeammates().end(); ++i) {
    if (i->getSpeedVector().isKnown()) {
      moveSpeed = i->getSpeedVector();
      if (i->getAcceleration().isKnown()) {
	moveSpeed += i->getAcceleration();
      }
      i->setPosition(i->getPosition() + moveSpeed);
      i->setSpeedVector(moveSpeed * facts->SP_player_decay);
    }
  }

  for (std::vector<PlayerObject>::iterator i = world->getOpponents().begin();
       i != world->getOpponents().end(); ++i) {
    if (i->getSpeedVector().isKnown()) {
      moveSpeed = i->getSpeedVector();
      if (i->getAcceleration().isKnown()) {
	moveSpeed += i->getAcceleration();
      }
      i->setPosition(i->getPosition() + moveSpeed);
      i->setSpeedVector(moveSpeed * facts->SP_player_decay);
    }
  }

  for (std::vector<PlayerObject>::iterator i = world->getUnknowns().begin();
       i != world->getUnknowns().end(); ++i) {
    if (i->getSpeedVector().isKnown()) {
      moveSpeed = i->getSpeedVector();
      if (i->getAcceleration().isKnown()) {
	moveSpeed += i->getAcceleration();
      }
      i->setPosition(i->getPosition() + moveSpeed);
      i->setSpeedVector(moveSpeed * facts->SP_player_decay);
    }
  }
	 
  
  // Clear Agent from movement.
  
  agent.setTurn(AngleDegUE());    
  agent.setTurnNeck(AngleDegUE());
  agent.setMovePosition(PointUnknown());
  agent.setAcceleration(VectorUE());
  ball.setAcceleration(VectorUE());
  
  world->setAgent(agent);
  world->setBall(ball);
  RS_LOG ( LA_STRATEGY, "BasicNewCycleStrategy - end");
}

RS_END_NAMESPACE

