/* -*- Mode: C++ -*-
 * $Id: KickBallView.h,v 1.6 2002/09/04 13:49:48 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: KickBallView.h
 * Unitname: Framework/view
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: s02davro $ $Date: 2002/09/04 13:49:48 $
 *
 *
 * HISTORY
 *
 * $Log: KickBallView.h,v $
 * Revision 1.6  2002/09/04 13:49:48  s02davro
 * *** empty log message ***
 *
 * Revision 1.5  2002/08/31 15:36:23  s02davro
 * Small modifications of views updated to the new world model
 *
 * Revision 1.4  2002/08/02 23:16:05  s02davro
 * Now works with the WorldModelInterface and new views
 *
 * Revision 1.3  2001/08/23 00:32:07  frehe
 * Removed dependencies.
 *
 * Revision 1.2  2000/10/28 20:22:03  mikbr212
 * changed initBall: now ballSpeed is retrieved from commandView if not known in ballView
 *
 * Revision 1.1  2000/10/07 14:29:26  mikbr212
 * created
 *
 * Revision 1.2  2000/09/17 14:17:32  frehe
 * Made minor updates to the KickBallView.
 *
 * Revision 1.1  2000/09/13 00:06:21  frehe
 * Added a view template called KickBallView.
 *
 */

/**
 * @file KickBallView.h
 * Include file for the view class KickBallView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.6 $
 * $Author: s02davro $
 * $Date: 2002/09/04 13:49:48 $
 */


#ifndef KICKBALLVIEW_H
#define KICKBALLVIEW_H


#include "View.h"
#include "CoordVector.h"
#include "CoordPoint.h"
#include "view_constants.h"
#include "WorldModelInterface.h"

RS_BEGIN_NAMESPACE

class BallView;
class AgentView;

/**
 * @class KickBallView KickBallView.h
 * A stub class for views that can be used as a template
 * for creating new views. It uses history.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.6 $
 * $Author: s02davro $
 * $Date: 2002/09/04 13:49:48 $
 */
// If you do not want any history then inherit from View instead.
//class KickBallView : public View {
class KickBallView : public View {
public:
  /** Construct a view with the ViewId @a id. */
  KickBallView(const ViewId id);

  /** Destroys the view. */
  ~KickBallView();

  FloatUE kickPowerRate();
  VectorUE getKickVector(const Vector& desBallSpeed,
			 Float maxKick = 100);
  bool canShootToSpeedVector(const Vector& desBallSpeed,
			     Float maxKick = -1.0);
protected:  
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();
  
  VectorUE getRelativeVector() const;
  const AgentObject& getAgent() const;
  const BallObject& getBall() const;
  const WorldFacts* facts;
private:
  // Hide copy-constructor and assignment.
  KickBallView(const KickBallView&);
  KickBallView& operator= (const KickBallView&);  
};

RS_END_NAMESPACE


#endif // !KICKBALLVIEW_H
