/* -*- Mode: C++ -*-
 * $Id: AgentObject.h,v 1.5 2002/08/31 14:46:01 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AgentObject.h
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2002/08/31 14:46:01 $
 *
 *
 * HISTORY
 *
 * $Log: AgentObject.h,v $
 * Revision 1.5  2002/08/31 14:46:01  s02davro
 * Small changes in the game objects.
 *
 * Revision 1.4  2002/08/02 21:28:51  s02davro
 * Changed some types, UCO->UE, Added some information about what is being done for the moment, See the attributes powerUsed, movePosition, turn and neckTurn for more information. Updated the documentation.
 *
 * Revision 1.3  2001/10/14 21:21:45  frehe
 * Added better support for messages from self and from other players.
 *
 * Revision 1.2  2000/10/18 23:05:36  frehe
 * Added lastCoachMessage and lastPlayerMessage.
 *
 * Revision 1.1  2000/09/01 03:24:30  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file AgentObject.h
 * Include file for the class AgentObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2002/08/31 14:46:01 $
 */

#ifndef AGENTOBJECT_H
#define AGENTOBJECT_H

#include "PlayerObject.h"

RS_BEGIN_NAMESPACE

/**
 * @class AgentObject AgentObject.h
 * Class used to represent the agent.
 *
 *
 * @todo Make a toString and operator<<.  
 * @todo Add information about other things the agent can do in the new 
 *       server (tackle, peek). 
 * @todo Add information for heterogeneous players. 
 *       
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2002/08/31 14:46:01 $
 */

class AgentObject : public PlayerObject {
public:
  /** Construct the AgentObject. Default with all values unknown. */
  AgentObject(const PointUE& def_point = PointUE(),
	      const TimeUnknown& def_time = TimeUnknown(),
	      const VectorUE& def_speed = VectorUE(),
	      const TimeUnknown& def_speed_time = TimeUnknown(),
	      const AngleDegUE& def_body_dir = AngleDegUE(),
	      const AngleDegUE& def_neck_dir = AngleDegUE(),
	      const Team def_team = T_UNKNOWN,
	      const Side def_side = S_UNKNOWN,
	      const TimeUnknown& def_team_time = TimeUnknown(),
	      const UIntUnknown def_u_number = UIntUnknown(),
	      const TimeUnknown& def_unum_time = TimeUnknown(),
	      const ViewWidth def_view_width = VW_UNKNOWN,
	      const ViewQuality def_view_quality = VQ_UNKNOWN,	      
	      const bool def_goalie = false,
	      const FloatUE def_stamina = FloatUE(),
	      const FloatUE def_effort = FloatUE(),
	      const FloatUE def_recover = FloatUE(),
	      const VectorUE& def_acceleration = VectorUE(),
	      const Float& def_power = Float(),
	      const PointUnknown& def_move = PointUnknown(),
	      const AngleDegUE& def_turn = AngleDegUE(),
	      const AngleDegUE& def_neckTurn = AngleDegUE());
  
  /** Copy constructor for the agent. */
  AgentObject(const AgentObject& obj);
  
  /** Assignment operator. */
  AgentObject& operator= (const AgentObject& obj);
  
  /** Set all agent attributes to unknown. */
  void reset();
  
  /** The agent is the goalie of the team. */
  void setGoalie(const bool g);

  /** Returns whether the agent is a goalie or not. */
  bool isGoalie() const;

  /** Set the agents stamina. */
  void setStamina(const FloatUE new_stamina);

  /** Returns the agents stamina. */
  FloatUE getStamina() const;

  /** Set the agents effort */
  void setEffort(const FloatUE new_effort);
  
  /** Returns the agents effort. */
  FloatUE getEffort() const;

  /** Set the agents recovery value. */
  void setRecover(const FloatUE new_recover);

  /** Returns the agent recovery value. */
  FloatUE getRecover() const;

  /** Set the number of dashes executed by the agent. */
  void setDashes(const UInt new_dashes);
  
  /** Returns the number of dashes executed by the agent. */
  UInt getDashes() const;

  /** Set the number of kicks executed by the agent. */
  void setKicks(const UInt new_kicks);

  /** Returns the number of kicks executed by the agent. */
  UInt getKicks() const;

  /** Set the number of says executed by the agent. */
  void setSays(const UInt new_says);

  /** Returns the number of says executed by the agent. */
  UInt getSays() const;

  /** Set the number of turns executed by the agent. */
  void setTurns(const UInt new_turns);

  /** Returns the number of turns executed by the agent. */
  UInt getTurns() const;

  /** Set the number of neckturns executed by the agent. */
  void setTurnNecks(const UInt new_turn_necks);

  /** Returns the number of neckturns executed by the agent. */
  UInt getTurnNecks() const;
  
  /** Set the number of cycles since the agent (goalie) made a catch. */
  void setCyclesSinceCatch(const UInt new_cycs_since_catch);
  
  /** Returns the cycles since last catch. */
  UInt getCyclesSinceCatch() const;
  
  /** Set the number of cycles since the agent last spoke. */
  void setCyclesSinceSay(const UInt new_cycs_since_say);

  /** Return the number of cycles since the agent last spoke. */
  UInt getCyclesSinceSay() const;
  
  /** Sets the agents ViewWidth. */
  void setViewWidth(const ViewWidth new_view_width);

  /** Gets the agents ViewWidth. */
  ViewWidth getViewWidth() const;
  
  /** Set the agents ViewQUality. */
  void setViewQuality(const ViewQuality new_view_quality);

  /** Gets the agents ViewQuality. */
  ViewQuality getViewQuality() const;

  /** Set the last message heard from coach. */
  void setCoachMessage(const std::string& msg);

  /** Returns the last message heard from coach. */
  const std::string& getCoachMessage() const;
  
  /** Set the last message heard from ourself. */
  void setSelfMessage(const std::string& msg);

  /** Returns the last message heard from ourself. */
  const std::string& getSelfMessage() const;
  
  /** Set the last message heard from another player. */
  void setPlayerMessage(const std::string& msg);

  /** Returns the last message heard from another player. */
  const std::string& getPlayerMessage() const;
  
  /** Set the direction of the last playermessage. */
  void setPlayerMessageDirection(const AngleDegUE& dir);

  /** Returns the direction of the last playermessage. */
  AngleDegUE getPlayerMessageDirection() const;
 
  /** Set the power currently used (for dashing) by the agent. */
  void setPowerUsed(const Float& power);

  /** Returns the power used at the moment. */
  Float getPowerUsed() const;

  /** Set the position the agent is about to move to. */
  void setMovePosition(const PointUnknown& def_move);

  /** Returns the position the agent is about to move to. */
  PointUnknown getMovePosition() const;

  /** Set the turn the agent is about to execute. */
  void setTurn(const AngleDegUE& power);
  
  /** Returns the turn the agent is about to execute. */
  AngleDegUE getTurn() const;
  
  /** Set the neckturn the agent is about to execute. */
  void setTurnNeck(const AngleDegUE& power);
  
  /** Returns the neckturn the agent is about to execute. */
  AngleDegUE getTurnNeck() const;
  
protected:
  /** Is the agent the goalie of the team? */
  bool goalie;
  
  /** Stamina for the agent. */
  FloatUE stamina;

  /** Effort for the agent. */
  FloatUE effort;

  /** Recovery for the agent. */
  FloatUE recover;

  /** Number of executed dashes. */
  UInt dashes;

  /** Number of executed kicks. */
  UInt kicks;

  /** Number of executed says. */
  UInt says;

  /** Number of executed turns. */
  UInt turns;

  /** Number of executed neckturns. */
  UInt turnNecks;

  /** The agens viewWidth.
   *  The valid values are VW_WIDE (90deg), VW_NORMAL (45deg)
   *  VW_NARROW (22.5deg). */
  ViewWidth viewWidth;

  /** The agents viewQuality.
   *  The valid values are VQ_HIGH (good), VQ_LOW (not recomended). */      
  ViewQuality viewQuality;
  
  /** The number of cycles since the agent made a catch. */
  UInt cyclesSinceCatch;
  
  /** The number of cycles since the agent spoke. */
  UInt cyclesSinceSay;

  /** The last message heard from the coach. */
  std::string coachMessage;

  /** The last message heard from ourself. */
  std::string selfMessage;

  /** The last message heard from another player. */ 
  std::string playerMessage;

  /** The direction of the last playerMessage. */
  AngleDegUE playerMessageDirection;

  /** The power currently used by the agent. */
  Float powerUsed;
  
  /** The position the agent is about to move to. */
  PointUnknown movePosition;
  
  /** The turn the agent is about to execute. */
  AngleDegUE turn;
  
  /** The neckturn the agent is about to execute. */
  AngleDegUE neckTurn;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "AgentObject.icc"
#endif


#endif // !AGENTOBJECT_H
