/* -*- Mode: C++ -*-
 * $Id: AttentionToCommand.cc,v 1.1 2002/10/01 16:06:28 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AttentionToCommand.cc
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: David Rosn 2002-09-30
 * Last modified by $Author: s02davro $ $Date: 2002/10/01 16:06:28 $
 *
 *
 * HISTORY
 *
 * $Log: AttentionToCommand.cc,v $
 * Revision 1.1  2002/10/01 16:06:28  s02davro
 * Added some new (version 8) commandsd
 *
 *
 */

/**
 * @file AttentionToCommand.cc
 * Implementation of the class AttentionToCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: s02davro $
 * $Date: 2002/10/01 16:06:28 $
 */


#include "AttentionToCommand.h"
#include "string_utils.h"


RS_BEGIN_NAMESPACE


AttentionToCommand::AttentionToCommand(const Team def_team, const UInt def_unumber)
  : ImmediateCommand(CMD_AttentionTo),
    team(def_team),
    uniformNumber(def_unumber),
    off(false) {
}

AttentionToCommand::AttentionToCommand()
  : ImmediateCommand(CMD_AttentionTo),
    off(true) {
}

AttentionToCommand::AttentionToCommand(const AttentionToCommand& obj)
  : ImmediateCommand(obj),
    team(obj.team),
    uniformNumber(obj.uniformNumber),
    off(obj.off) {
}

AttentionToCommand::~AttentionToCommand() {
}

void AttentionToCommand::copyCommand(const AttentionToCommand& obj) {
  ImmediateCommand::copyCommand(obj);
  team = obj.team;
  uniformNumber = obj.uniformNumber;
  off = obj.off;
}

AttentionToCommand& AttentionToCommand::operator=(const AttentionToCommand& obj) {
  if ( this != &obj ) {
    copyCommand(obj);
  }
  return *this;
}

const std::string AttentionToCommand::serverString() const {
  std::string tmp = "(attentionto ";
  if (off) 
    tmp += "off)";
  else {
    if (team == T_OUR_TEAM)
      tmp += "our ";
    else
      tmp += "opp ";
    tmp += toString(uniformNumber) + ")";
  }
  return tmp;
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "AttentionToCommand.icc"
#endif
