/* -*- Mode: C++ -*-
 * $Id: AttentionToCommand.h,v 1.1 2002/10/01 16:06:27 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AttentionToCommand.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: David Rosn 2002-09-30
 * Last modified by $Author: s02davro $ $Date: 2002/10/01 16:06:27 $
 *
 *
 * HISTORY
 *
 * $Log: AttentionToCommand.h,v $
 * Revision 1.1  2002/10/01 16:06:27  s02davro
 * Added some new (version 8) commandsd
 *
 *
 */

/**
 * @file AttentionToCommand.h
 * Include file for the class AttentionToCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: s02davro $
 * $Date: 2002/10/01 16:06:27 $
 */


#ifndef ATTENTIONTOCOMMAND_H
#define ATTENTIONTOCOMMAND_H

#include "ImmediateCommand.h"
#include "soccer_types.h"

RS_BEGIN_NAMESPACE

/**
 * @class AttentionToCommand AttentionToCommand.h
 * Class used to represent the server command (catch Direction).
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: s02davro $
 * $Date: 2002/10/01 16:06:27 $
 */

class AttentionToCommand : public ImmediateCommand {
public:
  AttentionToCommand(const Team def_team, const UInt def_unumber);
  AttentionToCommand();
  AttentionToCommand(const AttentionToCommand& obj);
  virtual ~AttentionToCommand();

  AttentionToCommand& operator =(const AttentionToCommand& obj);
  
  virtual bool onlyOneEachCycle() const;
  virtual const std::string serverString() const;
  
  Team getTeam() const;
  void setTeam(const Team def_team);
  UInt getUniformNumber() const;
  void setUniformNumber(const UInt def_unumber);  
  bool getOff() const;
  void setOff(const bool def_off);  

  /** Returns a new copy of the command.
    REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual AttentionToCommand* clone() const;
  virtual const AttentionToCommand* const getConstAttentionToCommand() const;
  virtual AttentionToCommand* getAttentionToCommand();

protected:
  void copyCommand(const AttentionToCommand& obj);

private:
  Team team;
  UInt uniformNumber;
  bool off;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "AttentionToCommand.icc"
#endif


#endif // !ATTENTIONTOCOMMAND_H
