/* -*- Mode: C++ -*-
 * $Id: PointToCommand.cc,v 1.1 2002/10/01 16:06:27 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PointToCommand.cc
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: David Rosn 2002-09-30
 * Last modified by $Author: s02davro $ $Date: 2002/10/01 16:06:27 $
 *
 *
 * HISTORY
 *
 * $Log: PointToCommand.cc,v $
 * Revision 1.1  2002/10/01 16:06:27  s02davro
 * Added some new (version 8) commandsd
 *
 *
 */

/**
 * @file PointToCommand.cc
 * Implementation of the class PointToCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: s02davro $
 * $Date: 2002/10/01 16:06:27 $
 */


#include "PointToCommand.h"
#include "string_utils.h"


RS_BEGIN_NAMESPACE


PointToCommand::PointToCommand(const Float& def_distance, const AngleDeg& def_direction)
  : PeriodicCommand(CMD_PointTo),
    distance(def_distance),
    direction(def_direction),
    off(false) {
}

PointToCommand::PointToCommand()
  : PeriodicCommand(CMD_AttentionTo),
    off(true) {
}

PointToCommand::PointToCommand(const PointToCommand& obj)
  : PeriodicCommand(obj),
    distance(obj.distance),
    direction(obj.direction),
    off(obj.off) {
}

PointToCommand::~PointToCommand() {
}

void PointToCommand::copyCommand(const PointToCommand& obj) {
  PeriodicCommand::copyCommand(obj);
  distance = obj.distance;
  direction = obj.direction;
  off = obj.off;
}

PointToCommand& PointToCommand::operator=(const PointToCommand& obj) {
  if ( this != &obj ) {
    copyCommand(obj);
  }
  return *this;
}

const std::string PointToCommand::serverString() const {
  std::string tmp = "(pointto ";
  if (off) 
    tmp += "off)";
  else {
    tmp += toString(distance) + " " + toString(direction.getDeg()) + ")";
  }
  return tmp;
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "PointToCommand.icc"
#endif
