/* -*- Mode: C++ -*-
 * $Id: PointToCommand.h,v 1.1 2002/10/01 16:06:27 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PointToCommand.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: David Rosn 2002-09-30
 * Last modified by $Author: s02davro $ $Date: 2002/10/01 16:06:27 $
 *
 *
 * HISTORY
 *
 * $Log: PointToCommand.h,v $
 * Revision 1.1  2002/10/01 16:06:27  s02davro
 * Added some new (version 8) commandsd
 *
 *
 */

/**
 * @file PointToCommand.h
 * Include file for the class PointToCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: s02davro $
 * $Date: 2002/10/01 16:06:27 $
 */


#ifndef POINTTOCOMMAND_H
#define POINTTOCOMMAND_H

#include "PeriodicCommand.h"
#include "library_types.h"

RS_BEGIN_NAMESPACE

/**
 * @class PointToCommand PointToCommand.h
 * Class used to represent the server command (pointto Distance Direction).
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: s02davro $
 * $Date: 2002/10/01 16:06:27 $
 */

class PointToCommand : public PeriodicCommand {
public:
  PointToCommand(const Float& def_distance, const AngleDeg& def_direction);
  PointToCommand();
  PointToCommand(const PointToCommand& obj);
  virtual ~PointToCommand();

  PointToCommand& operator =(const PointToCommand& obj);
  
  virtual bool onlyOneEachCycle() const;
  virtual const std::string serverString() const;
  
  Float getDistance() const;
  void setDistance(const Float& def_distance);
  AngleDeg getDirection() const;
  void setDirection(const AngleDeg& def_direction);  
  bool getOff() const;
  void setOff(const bool def_off);  
  
  /** Returns a new copy of the command.
    REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual PointToCommand* clone() const;
  virtual const PointToCommand* const getConstPointToCommand() const;
  virtual PointToCommand* getPointToCommand();

protected:
  void copyCommand(const PointToCommand& obj);

private:
  Float distance;
  AngleDeg direction;
  bool off;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "PointToCommand.icc"
#endif


#endif // !POINTTOCOMMAND_H
