/* -*- Mode: C++ -*-
 * $Id: TackleCommand.cc,v 1.1 2002/10/01 16:06:26 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TackleCommand.cc
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: David Rosn 2002-09-30
 * Last modified by $Author: s02davro $ $Date: 2002/10/01 16:06:26 $
 *
 *
 * HISTORY
 *
 * $Log: TackleCommand.cc,v $
 * Revision 1.1  2002/10/01 16:06:26  s02davro
 * Added some new (version 8) commandsd
 *
 *
 */

/**
 * @file TackleCommand.cc
 * Implementation of the class TackleCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: s02davro $
 * $Date: 2002/10/01 16:06:26 $
 */


#include "TackleCommand.h"
#include "string_utils.h"


RS_BEGIN_NAMESPACE


TackleCommand::TackleCommand(const AngleDeg& ang)
  : PeriodicCommand(CMD_Tackle),
    angle(ang) {
}


TackleCommand::TackleCommand(const TackleCommand& obj)
  : PeriodicCommand(obj),
    angle(obj.angle) {
}


TackleCommand::~TackleCommand() {
}


void TackleCommand::copyCommand(const TackleCommand& obj) {
  PeriodicCommand::copyCommand(obj);
  angle = obj.angle;
}


TackleCommand& TackleCommand::operator=(const TackleCommand& obj) {
  if ( this != &obj ) {
    copyCommand(obj);
  }
  return *this;
}


const std::string TackleCommand::serverString() const {
  return "(tackle " + toString(angle.getDeg()) + ")";
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "TackleCommand.icc"
#endif
