/**
 * @file SkillDependencies.h
 *
 * This file will be used to keep track
 * of the current dependencies between the skills
 * in the RoboSoc system.
 *
 */


/** Base class for all skills. */
class Skill {};


/** Base class for all with arguments. */
class SkillWithArguments : public Skill {};

/** Base class for all skills using a direction
    as a parameter. */
class SkillWithDirection : public SkillWithArguments {};


/** Base class for all skills using a position
    as a parameter. */
class SkillWithPosition : public SkillWithArguments {};


/** Base class for all skills using a speed
    as a parameter. */
class SkillWithSpeed : public SkillWithArguments {};


/** Base class for all skills using a vector
    as a parameter. */
class SkillWithVector : public SkillWithArguments {};


/** Skill to move the agent to a position Point(X, Y). */
class MoveToPosition : public Skill {
};


/** Skill used by a goal keeper to catch the ball. */
class CatchBall : public Skill {
};


/** Skill used by a player to find out where the ball is. */
class FindBall : public Skill {
};


/** Skill used by a player to go to the ball. */
class GotoBall : public Skill {
};


/** Skill used by a player to kick the ball to a certain speed vector. */
class KickBallToSpeedVector : public Skill {
};


/** Turn the agent to a specific direction. */
class TurnDirection : public Skill {
};
